@@ -27,6 +27,8 @@
 #include <celutil/debug.h>
 #include <celmath/mathlib.h>
 #include <celengine/astro.h>
+#include <celestia/celx.h>
+#include <celestia/celx_internal.h>
 #include "astro.h"
 #include "cmdparser.h"
 #include "glcontext.h"
@@ -37,6 +39,8 @@ using namespace std;
 static int parseRenderFlags(string);
 static int parseLabelFlags(string);
 static int parseOrbitFlags(string);
+static int parseConstellations(CommandConstellations* cmd, string s, int act);
+int parseConstellationColor(CommandConstellationColor* cmd, string s, Vec3d *col, int act);
 
 CommandParser::CommandParser(istream& in)
 {
@@ -116,24 +120,28 @@ void CommandParser::error(const string errMsg)
 }
 
 
-static astro::CoordinateSystem parseCoordinateSystem(const string& name)
+static ObserverFrame::CoordinateSystem parseCoordinateSystem(const string& name)
 {
+	// 'geographic' is a deprecated name for the bodyfixed coordinate system,
+	// maintained here for compatibility with older scripts.
     if (compareIgnoringCase(name, "observer") == 0)
-        return astro::ObserverLocal;
+        return ObserverFrame::ObserverLocal;
+    else if (compareIgnoringCase(name, "bodyfixed") == 0)
+        return ObserverFrame::BodyFixed;
     else if (compareIgnoringCase(name, "geographic") == 0)
-        return astro::Geographic;
+        return ObserverFrame::BodyFixed;
     else if (compareIgnoringCase(name, "equatorial") == 0)
-        return astro::Equatorial;
+        return ObserverFrame::Equatorial;
     else if (compareIgnoringCase(name, "ecliptical") == 0)
-        return astro::Ecliptical;
+        return ObserverFrame::Ecliptical;
     else if (compareIgnoringCase(name, "universal") == 0)
-        return astro::Universal;
+        return ObserverFrame::Universal;
     else if (compareIgnoringCase(name, "lock") == 0)
-        return astro::PhaseLock;
+        return ObserverFrame::PhaseLock;
     else if (compareIgnoringCase(name, "chase") == 0)
-        return astro::Chase;
+        return ObserverFrame::Chase;
     else
-        return astro::ObserverLocal;
+        return ObserverFrame::ObserverLocal;
 }
 
 
@@ -198,7 +206,7 @@ Command* CommandParser::parseCommand()
         string targetName;
         paramList->getString("target", targetName);
         string coordSysName;
-        astro::CoordinateSystem coordSys = astro::Universal;
+        ObserverFrame::CoordinateSystem coordSys = ObserverFrame::Universal;
         if (paramList->getString("coordsys", coordSysName))
             coordSys = parseCoordinateSystem(coordSysName);
 
@@ -217,7 +225,7 @@ Command* CommandParser::parseCommand()
         double distance = 5.0;
         paramList->getNumber("distance", distance);
 
-        astro::CoordinateSystem upFrame = astro::ObserverLocal;
+        ObserverFrame::CoordinateSystem upFrame = ObserverFrame::ObserverLocal;
         string frameString;
         if (paramList->getString("upframe", frameString))
             upFrame = parseCoordinateSystem(frameString);
@@ -544,6 +552,32 @@ Command* CommandParser::parseCommand()
             clearFlags = parseOrbitFlags(s);
 
         cmd = new CommandOrbitFlags(setFlags, clearFlags);
+    }
+	else if (commandName == "constellations")
+    {
+        string s;
+		CommandConstellations *cmdcons= new CommandConstellations();
+		
+        if (paramList->getString("set", s))
+            parseConstellations(cmdcons, s, 1);
+        if (paramList->getString("clear", s))
+            parseConstellations(cmdcons, s, 0);   
+		cmd = cmdcons;
+    }
+
+	else if (commandName == "constellationcolor")
+    {
+        string s;
+		CommandConstellationColor *cmdconcol= new CommandConstellationColor();
+		
+		Vec3d colorv(1.0f, 0.0f, 0.0f);
+        paramList->getVector("color", colorv);
+
+        if (paramList->getString("set", s))
+            parseConstellationColor(cmdconcol, s, &colorv, 1);
+        if (paramList->getString("clear", s))
+            parseConstellationColor(cmdconcol, s, &colorv, 0);   
+		cmd = cmdconcol;
     }
     else if (commandName == "setvisibilitylimit")
     {
@@ -563,6 +597,25 @@ Command* CommandParser::parseCommand()
         paramList->getNumber("brightness", brightness);
         cmd = new CommandSetAmbientLight((float) brightness);
     }
+    else if (commandName == "setgalaxylightgain")
+    {
+        double gain = 0.0;
+        paramList->getNumber("gain", gain);
+        cmd = new CommandSetGalaxyLightGain((float) gain);
+    }
+    else if (commandName == "settextureresolution")
+    {
+        string textureRes;
+        unsigned int res = 1;
+        paramList->getString("resolution", textureRes);
+        if (compareIgnoringCase(textureRes, "low") == 0)
+            res = 0;
+        else if (compareIgnoringCase(textureRes, "medium") == 0)
+            res = 1;
+        else if (compareIgnoringCase(textureRes, "high") == 0)
+            res = 2;
+        cmd = new CommandSetTextureResolution(res);
+    }
     else if (commandName == "preloadtex")
     {
         string object;
@@ -581,40 +634,43 @@ Command* CommandParser::parseCommand()
         paramList->getNumber("alpha", alpha);
         Color color((float) colorv.x, (float) colorv.y, (float) colorv.z, (float) alpha);
 
-        Marker::Symbol symbol = Marker::Diamond;
+        MarkerRepresentation rep(MarkerRepresentation::Diamond);
         string symbolString;
         if (paramList->getString("symbol", symbolString))
         {
             if (compareIgnoringCase(symbolString, "diamond") == 0)
-                symbol = Marker::Diamond;
+                rep = MarkerRepresentation(MarkerRepresentation::Diamond);
             else if (compareIgnoringCase(symbolString, "triangle") == 0)
-                symbol = Marker::Triangle;
+                rep = MarkerRepresentation(MarkerRepresentation::Triangle);
             else if (compareIgnoringCase(symbolString, "square") == 0)
-                symbol = Marker::Square;
+                rep = MarkerRepresentation(MarkerRepresentation::Square);
             else if (compareIgnoringCase(symbolString, "filledsquare") == 0)
-                symbol = Marker::FilledSquare;
+                rep = MarkerRepresentation(MarkerRepresentation::FilledSquare);
             else if (compareIgnoringCase(symbolString, "plus") == 0)
-                symbol = Marker::Plus;
+                rep = MarkerRepresentation(MarkerRepresentation::Plus);
             else if (compareIgnoringCase(symbolString, "x") == 0)
-                symbol = Marker::X;
+                rep = MarkerRepresentation(MarkerRepresentation::X);
             else if (compareIgnoringCase(symbolString, "leftarrow") == 0)
-                symbol = Marker::LeftArrow;
+                rep = MarkerRepresentation(MarkerRepresentation::LeftArrow);
             else if (compareIgnoringCase(symbolString, "rightarrow") == 0)
-                symbol = Marker::RightArrow;
+                rep = MarkerRepresentation(MarkerRepresentation::RightArrow);
             else if (compareIgnoringCase(symbolString, "uparrow") == 0)
-                symbol = Marker::UpArrow;
+                rep = MarkerRepresentation(MarkerRepresentation::UpArrow);
             else if (compareIgnoringCase(symbolString, "downarrow") == 0)
-                symbol = Marker::DownArrow;
+                rep = MarkerRepresentation(MarkerRepresentation::DownArrow);
             else if (compareIgnoringCase(symbolString, "circle") == 0)
-                symbol = Marker::Circle;
+                rep = MarkerRepresentation(MarkerRepresentation::Circle);
             else if (compareIgnoringCase(symbolString, "disk") == 0)
-                symbol = Marker::Disk;
+                rep = MarkerRepresentation(MarkerRepresentation::Disk);
         }
-
+        
         string label;
         paramList->getString("label", label);
+        rep.setSize((float) size);
+        rep.setColor(color);
+        rep.setLabel(label);        
 
-        cmd = new CommandMark(object, color, (float) size, symbol, label);
+        cmd = new CommandMark(object, rep);
     }
     else if (commandName == "unmark")
     {
@@ -653,6 +709,58 @@ Command* CommandParser::parseCommand()
 
         cmd = new CommandRenderPath(glcpath);
     }
+    else if (commandName == "splitview")
+    {
+        unsigned int view = 1;
+        paramList->getNumber("view", view);
+        string splitType;
+        paramList->getString("type", splitType);
+        double splitPos = 0.5;
+        paramList->getNumber("position", splitPos);
+        cmd = new CommandSplitView(view, splitType, (float) splitPos);
+    }
+    else if (commandName == "deleteview")
+    {
+        unsigned int view = 1;
+        paramList->getNumber("view", view);
+        cmd = new CommandDeleteView(view);
+    }
+    else if (commandName == "singleview")
+    {
+        cmd = new CommandSingleView();
+    }
+    else if (commandName == "setactiveview")
+    {
+        unsigned int view = 1;
+        paramList->getNumber("view", view);
+        cmd = new CommandSetActiveView(view);
+    }
+    else if (commandName == "setradius")
+    {
+        string object;
+        paramList->getString("object", object);
+        double radius = 1.0;
+        paramList->getNumber("radius", radius);
+        cmd = new CommandSetRadius(object, (float) radius);
+    }
+    else if (commandName == "setlinecolor")
+    {
+        string item;
+        paramList->getString("item", item);
+        Vec3d colorv(1.0f, 0.0f, 0.0f);
+        paramList->getVector("color", colorv);
+        Color color((float) colorv.x, (float) colorv.y, (float) colorv.z);
+        cmd = new CommandSetLineColor(item, color);
+    }
+    else if (commandName == "setlabelcolor")
+    {
+        string item;
+        paramList->getString("item", item);
+        Vec3d colorv(1.0f, 0.0f, 0.0f);
+        paramList->getVector("color", colorv);
+        Color color((float) colorv.x, (float) colorv.y, (float) colorv.z);
+        cmd = new CommandSetLabelColor(item, color);
+    }
     else
     {
         error("Unknown command name '" + commandName + "'");
@@ -678,54 +786,16 @@ int parseRenderFlags(string s)
         if (ttype == Tokenizer::TokenName)
         {
             string name = tokenizer.getNameValue();
-            if (compareIgnoringCase(name, "orbits") == 0)
-                flags |= Renderer::ShowOrbits;
-            else if (compareIgnoringCase(name, "cloudmaps") == 0)
-                flags |= Renderer::ShowCloudMaps;
-            else if (compareIgnoringCase(name, "constellations") == 0)
-                flags |= Renderer::ShowDiagrams;
-            else if (compareIgnoringCase(name, "galaxies") == 0)
-                flags |= Renderer::ShowGalaxies;
-            else if (compareIgnoringCase(name, "nebulae") == 0)
-                flags |= Renderer::ShowNebulae;
-            else if (compareIgnoringCase(name, "planets") == 0)
-                flags |= Renderer::ShowPlanets;
-            else if (compareIgnoringCase(name, "stars") == 0)
-                flags |= Renderer::ShowStars;
-	    else if (compareIgnoringCase(name, "nightmaps") == 0)
-		flags |= Renderer::ShowNightMaps;
-            else if (compareIgnoringCase(name, "eclipseshadows") == 0)
-                flags |= Renderer::ShowEclipseShadows;
-            else if (compareIgnoringCase(name, "ringshadows") == 0)
-                flags |= Renderer::ShowRingShadows;
-            else if (compareIgnoringCase(name, "comettails") == 0)
-                flags |= Renderer::ShowCometTails;
-            else if (compareIgnoringCase(name, "boundaries") == 0)
-                flags |= Renderer::ShowBoundaries;
-            else if (compareIgnoringCase(name, "markers") == 0)
-                flags |= Renderer::ShowMarkers;
-            else if (compareIgnoringCase(name, "automag") == 0)
-                flags |= Renderer::ShowAutoMag;
-            else if (compareIgnoringCase(name, "atmospheres") == 0)
-                flags |= Renderer::ShowAtmospheres;
-            else if (compareIgnoringCase(name, "grid") == 0)
-                flags |= Renderer::ShowCelestialSphere;
-            else if (compareIgnoringCase(name, "partialtrajectories") == 0)
-                flags |= Renderer::ShowPartialTrajectories;
-            else if (compareIgnoringCase(name, "openclusters") == 0)
-                flags |= Renderer::ShowOpenClusters;
-            else if (compareIgnoringCase(name, "cloudshadows") == 0)
-                flags |= Renderer::ShowCloudShadows;
+
+            if (CelxLua::RenderFlagMap.count(name) == 0)
+                cerr << "Unknown render flag: " << name << "\n";
+            else
+                flags |= CelxLua::RenderFlagMap[name];
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
                 ttype = tokenizer.nextToken();
         }
-        else
-        {
-            DPRINTF(0, "Command Parser: error parsing render flags\n");
-            return 0;
-        }
     }
 
     return flags;
@@ -745,40 +815,16 @@ int parseLabelFlags(string s)
         if (ttype == Tokenizer::TokenName)
         {
             string name = tokenizer.getNameValue();
-            if (compareIgnoringCase(name, "planets") == 0)
-                flags |= Renderer::PlanetLabels;
-            else if (compareIgnoringCase(name, "moons") == 0)
-                flags |= Renderer::MoonLabels;
-            else if (compareIgnoringCase(name, "spacecraft") == 0)
-                flags |= Renderer::SpacecraftLabels;
-            else if (compareIgnoringCase(name, "asteroids") == 0)
-                flags |= Renderer::AsteroidLabels;
-	    else if (compareIgnoringCase(name, "comets") == 0)
-		flags |= Renderer::CometLabels;
-            else if (compareIgnoringCase(name, "constellations") == 0)
-                flags |= Renderer::ConstellationLabels;
-            else if (compareIgnoringCase(name, "stars") == 0)
-                flags |= Renderer::StarLabels;
-	    else if (compareIgnoringCase(name, "galaxies") == 0)
-		flags |= Renderer::GalaxyLabels;
-	    else if (compareIgnoringCase(name, "locations") == 0)
-		flags |= Renderer::LocationLabels;
-	    else if (compareIgnoringCase(name, "nebulae") == 0)
-		flags |= Renderer::NebulaLabels;
-	    else if (compareIgnoringCase(name, "openclusters") == 0)
-		flags |= Renderer::OpenClusterLabels;
-	    else if (compareIgnoringCase(name, "i18nconstellations") == 0)
-		flags |= Renderer::I18nConstellationLabels;
+
+            if (CelxLua::LabelFlagMap.count(name) == 0)
+                cerr << "Unknown label flag: " << name << "\n";
+            else
+                flags |= CelxLua::LabelFlagMap[name];
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
                 ttype = tokenizer.nextToken();
         }
-        else
-        {
-            DPRINTF(0, "Command Parser: error parsing label flags\n");
-            return 0;
-        }
     }
 
     return flags;
@@ -798,16 +844,42 @@ int parseOrbitFlags(string s)
         if (ttype == Tokenizer::TokenName)
         {
             string name = tokenizer.getNameValue();
-            if (compareIgnoringCase(name, "planet") == 0)
-                flags |= Body::Planet;
-            else if (compareIgnoringCase(name, "moon") == 0)
-                flags |= Body::Moon;
-            else if (compareIgnoringCase(name, "asteroid") == 0)
-                flags |= Body::Asteroid;
-            else if (compareIgnoringCase(name, "comet") == 0)
-                flags |= Body::Comet;
-            else if (compareIgnoringCase(name, "spacecraft") == 0)
-                flags |= Body::Spacecraft;
+            name[0] = toupper(name[0]);
+
+            if (CelxLua::BodyTypeMap.count(name) == 0)
+                cerr << "Unknown orbit flag: " << name << "\n";
+            else
+                flags |= CelxLua::BodyTypeMap[name];
+
+            ttype = tokenizer.nextToken();
+            if (ttype == Tokenizer::TokenBar)
+                ttype = tokenizer.nextToken();
+        }
+    }
+
+    return flags;
+}
+
+
+int parseConstellations(CommandConstellations* cmd, string s, int act)
+{
+    istringstream in(s);
+
+    Tokenizer tokenizer(&in);
+    int flags = 0;
+
+    Tokenizer::TokenType ttype = tokenizer.nextToken();
+    while (ttype != Tokenizer::TokenEnd)
+    {
+        if (ttype == Tokenizer::TokenName)
+        {
+            string name = tokenizer.getNameValue();
+			if (compareIgnoringCase(name, "all") == 0 && act==1)
+				cmd->all=1;
+			else if (compareIgnoringCase(name, "all") == 0 && act==0)
+				cmd->none=1;
+			else 
+				cmd->setValues(name,act);
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
@@ -815,7 +887,46 @@ int parseOrbitFlags(string s)
         }
         else
         {
-            DPRINTF(0, "Command Parser: error parsing orbit flags\n");
+            DPRINTF(0, "Command Parser: error parsing render flags\n");
+            return 0;
+        }
+    }
+
+    return flags;
+}
+
+int parseConstellationColor(CommandConstellationColor* cmd, string s, Vec3d *col, int act)
+{
+    istringstream in(s);
+
+    Tokenizer tokenizer(&in);
+    int flags = 0;
+
+	if(!act)
+		cmd->unsetColor();
+	else
+		cmd->setColor((float)col->x, (float)col->y, (float)col->z);
+
+    Tokenizer::TokenType ttype = tokenizer.nextToken();
+    while (ttype != Tokenizer::TokenEnd)
+    {
+        if (ttype == Tokenizer::TokenName)
+        {
+            string name = tokenizer.getNameValue();
+			if (compareIgnoringCase(name, "all") == 0 && act==1)
+				cmd->all=1;
+			else if (compareIgnoringCase(name, "all") == 0 && act==0)
+				cmd->none=1;
+			else 
+				cmd->setConstellations(name);
+				
+            ttype = tokenizer.nextToken();
+            if (ttype == Tokenizer::TokenBar)
+                ttype = tokenizer.nextToken();
+        }
+        else
+        {
+            DPRINTF(0, "Command Parser: error parsing render flags\n");
             return 0;
         }
     }
