@@ -9,12 +9,12 @@
 
 #include <fstream>
 
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 #define JPEG_SUPPORT
 #define PNG_SUPPORT
 #endif
 
-#ifdef MACOSX
+#ifdef TARGET_OS_MAC
 #include <unistd.h>
 #include "CGBuffer.h"
 #ifndef PNG_SUPPORT
@@ -24,9 +24,9 @@
 #endif
 
 #ifndef _WIN32
-#ifndef MACOSX
+#ifndef TARGET_OS_MAC
 #include <config.h>
-#endif /* ! MACOSX */
+#endif /* ! TARGET_OS_MAC */
 #endif /* ! _WIN32 */
 
 #include "image.h"
@@ -49,11 +49,11 @@ extern "C" {
 #endif // JPEG_SUPPORT
 
 #ifdef PNG_SUPPORT // PNG_SUPPORT
-#ifdef MACOSX
+#ifdef TARGET_OS_MAC
 #include "../../macosx/png.h"
 #else
 #include "png.h"
-#endif // MACOSX
+#endif // TARGET_OS_MAC
 
 #include <celutil/debug.h>
 #include <celutil/util.h>
@@ -85,6 +85,13 @@ using namespace std;
 #endif // PNG_SUPPORT
 
 
+// All rows are padded to a size that's a multiple of 4 bytes
+static int pad(int n)
+{
+    return (n + 3) & ~0x3;
+}
+
+
 static int formatComponents(int fmt)
 {
     switch (fmt)
@@ -131,7 +138,7 @@ static int calcMipLevelSize(int fmt, int w, int h, int mip)
         // 4x4 blocks, 16 bytes per block
         return ((w + 3) / 4) * ((h + 3) / 4) * 16;
     default:
-        return h * w * formatComponents(fmt);
+        return h * pad(w * formatComponents(fmt));
     }
 }
 
@@ -139,14 +146,16 @@ static int calcMipLevelSize(int fmt, int w, int h, int mip)
 Image::Image(int fmt, int w, int h, int mips) :
     width(w),
     height(h),
-    mipLevels(mips), 
+    mipLevels(mips),
     format(fmt),
     pixels(NULL)
 {
     components = formatComponents(fmt);
     assert(components != 0);
 
-    size = 0;
+    pitch = pad(w * components);
+
+    size = 1;
     for (int i = 0; i < mipLevels; i++)
         size += calcMipLevelSize(fmt, w, h, i);
     pixels = new unsigned char[size];
@@ -172,6 +181,12 @@ int Image::getHeight() const
 }
 
 
+int Image::getPitch() const
+{
+    return pitch;
+}
+
+
 int Image::getMipLevelCount() const
 {
     return mipLevels;
@@ -204,7 +219,7 @@ unsigned char* Image::getPixels()
 
 unsigned char* Image::getPixelRow(int mip, int row)
 {
-    int w = max(width >> mip, 1);
+    /*int w = max(width >> mip, 1); Unused*/
     int h = max(height >> mip, 1);
     if (mip >= mipLevels || row >= h)
         return NULL;
@@ -213,7 +228,7 @@ unsigned char* Image::getPixelRow(int mip, int row)
     if (isCompressed())
         return NULL;
 
-    return getMipLevel(mip) + row * w * components;
+    return getMipLevel(mip) + row * pitch;
 }
 
 
@@ -292,6 +307,7 @@ Image* Image::computeNormalMap(float scale, bool wrap) const
         return NULL;
 
     unsigned char* nmPixels = normalMap->getPixels();
+    int nmPitch = normalMap->getPitch();
 
     // Compute normals using differences between adjacent texels.
     for (int i = 0; i < height; i++)
@@ -312,7 +328,7 @@ Image* Image::computeNormalMap(float scale, bool wrap) const
                 {
                     i0++;
                     i1++;
-                }   
+                }
             }
             if (j1 < 0)
             {
@@ -327,17 +343,17 @@ Image* Image::computeNormalMap(float scale, bool wrap) const
                 }
             }
 
-            int h00 = (int) pixels[(i0 * width + j0) * components];
-            int h10 = (int) pixels[(i0 * width + j1) * components];
-            int h01 = (int) pixels[(i1 * width + j0) * components];
-            
+            int h00 = (int) pixels[i0 * pitch + j0 * components];
+            int h10 = (int) pixels[i0 * pitch + j1 * components];
+            int h01 = (int) pixels[i1 * pitch + j0 * components];
+
             float dx = (float) (h10 - h00) * (1.0f / 255.0f) * scale;
             float dy = (float) (h01 - h00) * (1.0f / 255.0f) * scale;
 
             float mag = (float) sqrt(dx * dx + dy * dy + 1.0f);
             float rmag = 1.0f / mag;
 
-            int n = (i * width + j) * 4;
+            int n = i * nmPitch + j * 4;
             nmPixels[n]     = (unsigned char) (128 + 127 * dx * rmag);
             nmPixels[n + 1] = (unsigned char) (128 + 127 * dy * rmag);
             nmPixels[n + 2] = (unsigned char) (128 + 127 * rmag);
@@ -368,6 +384,7 @@ Image* LoadImageFromFile(const string& filename)
         img = LoadPNGImage(filename);
         break;
     case Content_DDS:
+    case Content_DXT5NormalMap:
         img = LoadDDSImage(filename);
         break;
     default:
@@ -407,7 +424,7 @@ METHODDEF(void) my_error_exit(j_common_ptr cinfo)
 #endif // JPEG_SUPPORT
 
 
-Image* LoadJPEGImage(const string& filename, int channels)
+Image* LoadJPEGImage(const string& filename, int)
 {
 #ifdef JPEG_SUPPORT
     Image* img = NULL;
@@ -536,8 +553,8 @@ Image* LoadJPEGImage(const string& filename, int channels)
     // warnings occurred (test whether jerr.pub.num_warnings is nonzero).
 
     return img;
-    
-#elif MACOSX
+
+#elif TARGET_OS_MAC
 
     Image* img = NULL;
     CGBuffer* cgJpegImage;
@@ -559,7 +576,7 @@ Image* LoadJPEGImage(const string& filename, int channels)
         delete cgJpegImage;
         return NULL;
     }
-    
+
     cgJpegImage->Render();
 
     img_w = (size_t) cgJpegImage->image_size.width;
@@ -591,17 +608,17 @@ Image* LoadJPEGImage(const string& filename, int channels)
         if ( (i % (img_w * img_d)) == 0 ) bin -= 2*(img_w * img_d);
 
 #ifndef MACOSX_ALPHA_JPEGS
-        if (( (img_d != 1) && !((i&3)^3) )) // skip extra byte 
+        if (( (img_d != 1) && !((i&3)^3) )) // skip extra byte
         {
             ++bin;
-        } else 
-#endif // !MACOSX_ALPHA_JPEGS         
+        } else
+#endif // !MACOSX_ALPHA_JPEGS
 
         *bout++ = *bin++;
-    }    
+    }
     delete cgJpegImage;
     return img;
-    
+
 #else
     return NULL;
 #endif // JPEG_SUPPORT
@@ -637,7 +654,7 @@ Image* LoadPNGImage(const string& filename)
         clog << _("Error opening image file ") << filename << '\n';
         return NULL;
     }
-   
+
     fread(header, 1, sizeof(header), fp);
     if (png_sig_cmp((unsigned char*) header, 0, sizeof(header)))
     {
@@ -661,7 +678,7 @@ Image* LoadPNGImage(const string& filename)
         png_destroy_read_struct(&png_ptr, (png_infopp) NULL, (png_infopp) NULL);
         return NULL;
     }
-   
+
     if (setjmp(png_jmpbuf(png_ptr)))
     {
         fclose(fp);
@@ -881,7 +898,7 @@ static Image* LoadBMPImage(ifstream& in)
                 }
             }
             break;
-            
+
         case 24:
             {
                 for (int x = 0; x < imageHeader.width; x++)
