@@ -1,6 +1,6 @@
 // mesh.cpp
 //
-// Copyright (C) 2004, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2004-2006, Chris Laurel <claurel@shatters.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -13,10 +13,12 @@
 #include "glext.h"
 #include <cassert>
 #include <iostream>
+#include <algorithm>
+
 using namespace std;
 
 
-static size_t VertexAttributeFormatSizes[Mesh::FormatMax] = 
+static size_t VertexAttributeFormatSizes[Mesh::FormatMax] =
 {
      4,  // Float1
      8,  // Float2
@@ -50,11 +52,12 @@ Mesh::Material::Material() :
     emissive(0.0f, 0.0f, 0.0f),
     specular(0.0f, 0.0f, 0.0f),
     specularPower(1.0f),
-    opacity(1.0f)
+    opacity(1.0f),
+    blend(NormalBlend)
 {
     for (int i = 0; i < TextureSemanticMax; i++)
         maps[i] = InvalidResource;
-        
+
 }
 
 
@@ -170,6 +173,30 @@ Mesh::PrimitiveGroup::~PrimitiveGroup()
 }
 
 
+uint32
+Mesh::PrimitiveGroup::getPrimitiveCount() const
+{
+    switch (prim)
+    {
+    case TriList:
+        return nIndices / 3;
+    case TriStrip:
+    case TriFan:
+        return nIndices - 2;
+    case LineList:
+        return nIndices / 2;
+    case LineStrip:
+        return nIndices - 2;
+    case PointList:
+    case SpriteList:
+        return nIndices;
+    default:
+        // Invalid value
+        return 0;
+    }
+}
+
+
 Mesh::Mesh() :
     vertexDesc(0, 0, NULL),
     nVertices(0),
@@ -188,8 +215,10 @@ Mesh::~Mesh()
         delete *iter;
     }
 
+    // TODO: this is just to cast away void* and shut up GCC warnings;
+    // should probably be static_cast<VertexList::VertexPart*>
     if (vertices != NULL)
-        delete vertices;
+        delete[] static_cast<char*>(vertices);
 
     if (vbObject != 0)
     {
@@ -253,11 +282,18 @@ Mesh::addGroup(PrimitiveGroupType prim,
     g->materialIndex = materialIndex;
     g->nIndices = nIndices;
     g->indices = indices;
-    
+
     return addGroup(g);
 }
 
 
+uint32
+Mesh::getGroupCount() const
+{
+    return groups.size();
+}
+
+
 void
 Mesh::clearGroups()
 {
@@ -300,6 +336,37 @@ Mesh::remapIndices(const vector<uint32>& indexMap)
 }
 
 
+void
+Mesh::remapMaterials(const vector<uint32>& materialMap)
+{
+    for (vector<PrimitiveGroup*>::iterator iter = groups.begin();
+         iter != groups.end(); iter++)
+    {
+        (*iter)->materialIndex = materialMap[(*iter)->materialIndex];
+    }
+}
+
+
+class PrimitiveGroupComparator : public std::binary_function<const Mesh::PrimitiveGroup*, const Mesh::PrimitiveGroup*, bool>
+{
+public:
+    bool operator()(const Mesh::PrimitiveGroup* g0, const Mesh::PrimitiveGroup* g1) const
+    {
+        return g0->materialIndex < g1->materialIndex;
+    }
+
+private:
+    int unused;
+};
+
+
+void
+Mesh::aggregateByMaterial()
+{
+    sort(groups.begin(), groups.end(), PrimitiveGroupComparator());
+}
+
+
 bool
 Mesh::pick(const Ray3d& ray, double& distance) const
 {
@@ -324,7 +391,7 @@ Mesh::pick(const Ray3d& ray, double& distance) const
         Mesh::PrimitiveGroupType primType = (*iter)->prim;
         uint32 nIndices = (*iter)->nIndices;
 
-        // Only attempt to compute the intersection of the ray with triangl
+        // Only attempt to compute the intersection of the ray with triangle
         // groups.
         if ((primType == TriList || primType == TriStrip || primType == TriFan) &&
             (nIndices >= 3) &&
@@ -334,7 +401,7 @@ Mesh::pick(const Ray3d& ray, double& distance) const
             uint32 i0 = (*iter)->indices[0];
             uint32 i1 = (*iter)->indices[1];
             uint32 i2 = (*iter)->indices[2];
-            
+
             // Iterate over the triangles in the primitive group
             do
             {
@@ -470,7 +537,7 @@ Mesh::render(const std::vector<const Material*>& materials,
         rc.setVertexArrays(vertexDesc, vertices);
     }
 
-    uint32 lastMaterial = ~0;
+    uint32 lastMaterial = ~0u;
 
     // Iterate over all primitive groups in the mesh
     for (vector<PrimitiveGroup*>::const_iterator iter = groups.begin();
@@ -501,9 +568,29 @@ Mesh::getBoundingBox() const
         return bbox;
 
     char* vdata = reinterpret_cast<char*>(vertices) + vertexDesc.getAttribute(Position).offset;
-    
-    for (uint32 i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
-        bbox.include(Point3f(reinterpret_cast<float*>(vdata)));
+
+    if (vertexDesc.getAttribute(PointSize).format == Float1)
+    {
+        // Handle bounding box calculation for point sprites. Unlike other
+        // primitives, point sprite vertices have a non-zero size.
+        int pointSizeOffset = (int) vertexDesc.getAttribute(PointSize).offset -
+            (int) vertexDesc.getAttribute(Position).offset;
+        
+        for (uint32 i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
+        {
+            Point3f center = Point3f(reinterpret_cast<float*>(vdata));
+            float pointSize = (reinterpret_cast<float*>(vdata + pointSizeOffset))[0];
+            Vec3f offsetVec(pointSize, pointSize, pointSize);
+
+            AxisAlignedBox pointbox(center - offsetVec, center + offsetVec);
+            bbox.include(pointbox);
+        }
+    }
+    else
+    {
+        for (uint32 i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
+            bbox.include(Point3f(reinterpret_cast<float*>(vdata)));
+    }
 
     return bbox;
 }
@@ -516,13 +603,39 @@ Mesh::transform(Vec3f translation, float scale)
         return;
 
     char* vdata = reinterpret_cast<char*>(vertices) + vertexDesc.getAttribute(Position).offset;
-    for (uint32 i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
+    uint32 i;
+
+    // Scale and translate the vertex positions
+    for (i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
     {
         Vec3f tv = (Vec3f(reinterpret_cast<float*>(vdata)) + translation) * scale;
         reinterpret_cast<float*>(vdata)[0] = tv.x;
         reinterpret_cast<float*>(vdata)[1] = tv.y;
         reinterpret_cast<float*>(vdata)[2] = tv.z;
     }
+
+    // Point sizes need to be scaled as well
+    if (vertexDesc.getAttribute(PointSize).format == Float1)
+    {
+        vdata = reinterpret_cast<char*>(vertices) + vertexDesc.getAttribute(PointSize).offset;
+        for (i = 0; i < nVertices; i++, vdata += vertexDesc.stride)
+            reinterpret_cast<float*>(vdata)[0] *= scale;
+    }
+}
+
+
+uint32
+Mesh::getPrimitiveCount() const
+{
+    uint32 count = 0;
+
+    for (vector<PrimitiveGroup*>::const_iterator iter = groups.begin();
+         iter != groups.end(); iter++)
+    {
+        count += (*iter)->getPrimitiveCount();
+    }
+
+    return count;
 }
 
 
@@ -541,6 +654,8 @@ Mesh::parsePrimitiveGroupType(const string& name)
         return LineStrip;
     else if (name == "points")
         return PointList;
+    else if (name == "sprites")
+        return SpriteList;
     else
         return InvalidPrimitiveGroupType;
 }
@@ -567,6 +682,8 @@ Mesh::parseVertexAttributeSemantic(const string& name)
         return Texture2;
     else if (name == "texcoord3")
         return Texture3;
+    else if (name == "pointsize")
+        return PointSize;
     else
         return InvalidSemantic;
 }
