@@ -19,11 +19,13 @@
 #include "parser.h"
 
 class ReferenceFrame;
+class TwoVectorFrame;
 class Universe;
+class Selection;
 
 bool ParseDate(Hash* hash, const string& name, double& jd);
 
-Orbit* CreateOrbit(PlanetarySystem* system,
+Orbit* CreateOrbit(const Selection& centralObject,
                    Hash* planetData,
                    const std::string& path,
                    bool usePlanetUnits);
@@ -35,6 +37,13 @@ RotationModel* CreateRotationModel(Hash* rotationData,
 RotationModel* CreateDefaultRotationModel(double syncRotationPeriod);
 
 ReferenceFrame* CreateReferenceFrame(const Universe& universe,
-                                     Value* frameValue);
+                                     Value* frameValue,
+                                     const Selection& defaultCenter,
+                                     Body* defaultObserver);
+
+TwoVectorFrame* CreateTopocentricFrame(const Selection& center,
+                                       const Selection& target,
+                                       const Selection& observer);
+
 
 #endif // _CELENGINE_PARSEOBJECT_H_
