@@ -1,6 +1,6 @@
 // render.h
 //
-// Copyright (C) 2001-2007, Celestia Development Team
+// Copyright (C) 2001-2008, Celestia Development Team
 // Contact: Chris Laurel <claurel@gmail.com>
 //
 // This program is free software; you can redistribute it and/or
@@ -25,10 +25,12 @@
 
 
 class RendererWatcher;
+class FrameTree;
+class ReferenceMark;
 
 struct LightSource
 {
-    Point3d position;
+    Vec3d position;
     Color color;
     float luminosity;
     float radius;
@@ -42,14 +44,16 @@ struct RenderListEntry
         RenderableStar,
         RenderableBody,
         RenderableCometTail,
-        RenderableBodyAxes,
-        RenderableFrameAxes,
-        RenderableSunDirection,
-        RenderableVelocityVector,
+        RenderableReferenceMark,
+    };
+
+    union
+    {
+        const Star* star;
+        Body* body;
+        const ReferenceMark* refMark;
     };
 
-    const Star* star;
-    Body* body;
     Point3f position;
     Vec3f sun;
     float distance;
@@ -61,7 +65,16 @@ struct RenderListEntry
     float appMag;
     RenderableType renderableType;
     bool isOpaque;
-    std::vector<LightSource>* lightSourceList;
+    //std::vector<LightSource>* lightSourceList;
+};
+
+
+struct SecondaryIlluminator
+{
+    const Body* body;
+    Vec3d position_v;       // viewer relative position
+    float radius;           // radius in km
+    float reflectedIrradiance;  // albedo times total irradiance from direct sources
 };
 
 
@@ -86,7 +99,7 @@ class Renderer
 
     float calcPixelSize(float fov, float windowHeight);
     void setFaintestAM45deg(float);
-    float getFaintestAM45deg();
+    float getFaintestAM45deg() const;
 
     void setRenderMode(int);
     void autoMag(float& faintestMag);
@@ -94,7 +107,11 @@ class Renderer
                 const Universe&,
                 float faintestVisible,
                 const Selection& sel);
-    
+    void draw(const Observer&,
+              const Universe&,
+              float faintestVisible,
+              const Selection& sel);
+
     enum {
         NoLabels            = 0x000,
         StarLabels          = 0x001,
@@ -109,32 +126,40 @@ class Renderer
         NebulaLabels        = 0x200,
         OpenClusterLabels   = 0x400,
         I18nConstellationLabels = 0x800,
-        BodyLabelMask       = (PlanetLabels | MoonLabels | AsteroidLabels | SpacecraftLabels | CometLabels),
-    };
+        DwarfPlanetLabels   = 0x1000,
+        MinorMoonLabels     = 0x2000,
+		GlobularLabels      = 0x4000,		        
+		BodyLabelMask       = (PlanetLabels | DwarfPlanetLabels | MoonLabels | MinorMoonLabels | AsteroidLabels | SpacecraftLabels | CometLabels),	   
+	};
 
     enum {
-        ShowNothing         = 0x0000,
-        ShowStars           = 0x0001,
-        ShowPlanets         = 0x0002,
-        ShowGalaxies        = 0x0004,
-        ShowDiagrams        = 0x0008,
-        ShowCloudMaps       = 0x0010,
-        ShowOrbits          = 0x0020,
-        ShowCelestialSphere = 0x0040,
-        ShowNightMaps       = 0x0080,
-        ShowAtmospheres     = 0x0100,
-        ShowSmoothLines     = 0x0200,
-        ShowEclipseShadows  = 0x0400,
-        ShowStarsAsPoints   = 0x0800,
-        ShowRingShadows     = 0x1000,
-        ShowBoundaries      = 0x2000,
-        ShowAutoMag         = 0x4000,
-        ShowCometTails      = 0x8000,
-        ShowMarkers         = 0x10000,
+        ShowNothing         =   0x0000,
+        ShowStars           =   0x0001,
+        ShowPlanets         =   0x0002,
+        ShowGalaxies        =   0x0004,
+        ShowDiagrams        =   0x0008,
+        ShowCloudMaps       =   0x0010,
+        ShowOrbits          =   0x0020,
+        ShowCelestialSphere =   0x0040,
+        ShowNightMaps       =   0x0080,
+        ShowAtmospheres     =   0x0100,
+        ShowSmoothLines     =   0x0200,
+        ShowEclipseShadows  =   0x0400,
+        ShowStarsAsPoints   =   0x0800,
+        ShowRingShadows     =   0x1000,
+        ShowBoundaries      =   0x2000,
+        ShowAutoMag         =   0x4000,
+        ShowCometTails      =   0x8000,
+        ShowMarkers         =  0x10000,
         ShowPartialTrajectories = 0x20000,
-        ShowNebulae         = 0x40000,
-        ShowOpenClusters    = 0x80000,
-        ShowCloudShadows    = 0x200000,
+        ShowNebulae         =  0x40000,
+        ShowOpenClusters    =  0x80000,
+		ShowGlobulars       =  0x100000,			        
+		ShowCloudShadows    =  0x200000,
+        ShowGalacticGrid    =  0x400000,
+        ShowEclipticGrid    =  0x800000,
+        ShowHorizonGrid     = 0x1000000,
+        ShowEcliptic        = 0x2000000,
     };
 
     enum StarStyle 
@@ -149,6 +174,7 @@ class Renderer
     static const int DefaultRenderFlags = Renderer::ShowStars          |
                                           Renderer::ShowPlanets        |
                                           Renderer::ShowGalaxies       |
+										  Renderer::ShowGlobulars      |	 
                                           Renderer::ShowCloudMaps      |
                                           Renderer::ShowAtmospheres    |
                                           Renderer::ShowEclipseShadows |
@@ -187,29 +213,84 @@ class Renderer
     void setVertexShaderEnabled(bool);
     bool vertexShaderSupported() const;
 
+#ifdef USE_HDR
+    bool getBloomEnabled();
+    void setBloomEnabled(bool);
+    void increaseBrightness();
+    void decreaseBrightness();
+    float getBrightness();
+#endif
+
     GLContext* getGLContext() { return context; }
 
     void setStarStyle(StarStyle);
     StarStyle getStarStyle() const;
     void setResolution(unsigned int resolution);
-    unsigned int getResolution();
+    unsigned int getResolution() const;
 
     void loadTextures(Body*);
 
     // Label related methods
-    static const int MaxLabelLength = 32;
-    struct Label
+    enum LabelAlignment
     {
-        char text[MaxLabelLength];
-        Color color;
-        Point3f position;
+        AlignCenter,
+        AlignLeft,
+        AlignRight
     };
     
-    void addLabel(const char* text, Color, const Point3f&, float depth = -1);
-    void addLabel(const std::string&, Color, const Point3f&, float depth = -1);
-    void addSortedLabel(const std::string&, Color, const Point3f&);
-    void clearLabels();
-	void clearSortedLabels();
+    enum LabelVerticalAlignment
+    {
+        VerticalAlignCenter,
+        VerticalAlignBottom,
+        VerticalAlignTop,
+    };
+        
+    static const int MaxLabelLength = 32;
+    struct Annotation
+    {
+        char labelText[MaxLabelLength];
+        const MarkerRepresentation* markerRep;
+        Color color;
+        Point3f position;
+        LabelAlignment halign : 3;
+        LabelVerticalAlignment valign : 3;
+        float size;
+
+        bool operator<(const Annotation&) const;
+    };
+        
+    void addForegroundAnnotation(const MarkerRepresentation* markerRep,
+                                 const std::string& labelText,
+                                 Color color,
+                                 const Point3f& position,
+                                 LabelAlignment halign = AlignLeft,
+                                 LabelVerticalAlignment valign = VerticalAlignBottom,
+                                 float size = 0.0f);
+    void addBackgroundAnnotation(const MarkerRepresentation* markerRep,
+                                 const std::string& labelText,
+                                 Color color,
+                                 const Point3f& position,
+                                 LabelAlignment halign = AlignLeft,
+                                 LabelVerticalAlignment valign = VerticalAlignBottom,
+                                 float size = 0.0f);
+    void addSortedAnnotation(const MarkerRepresentation* markerRep,
+                             const std::string& labelText,
+                             Color color,
+                             const Point3f& position,
+                             LabelAlignment halign = AlignLeft,
+                             LabelVerticalAlignment valign = VerticalAlignBottom,
+                             float size = 0.0f);
+
+    // Callbacks for renderables; these belong in a special renderer interface
+    // only visible in object's render methods.
+    void beginObjectAnnotations();
+    void addObjectAnnotation(const MarkerRepresentation* markerRep, const std::string& labelText, Color, const Point3f&);
+    void endObjectAnnotations();
+    Quatf getCameraOrientation() const;
+    float getNearPlaneDistance() const;
+    
+    void clearAnnotations(std::vector<Annotation>&);
+	void clearSortedAnnotations();
 
     void invalidateOrbitCache();
     
@@ -221,6 +302,8 @@ class Renderer
         const Star* star;
         Point3f origin;
         float opacity;
+
+        bool operator<(const OrbitPathListEntry&) const;
     };        
 
     enum FontStyle
@@ -259,21 +342,21 @@ class Renderer
             atmosphere(NULL),
             rings(NULL),
             radius(1.0f),
+            geometryScale(1.0f),
             semiAxes(1.0f, 1.0f, 1.0f),
-            model(InvalidResource),
-            orientation(1.0f),
-            locations(NULL)
+            geometry(InvalidResource),
+            orientation(1.0f)
         {};
 
         Surface* surface;
         const Atmosphere* atmosphere;
         RingSystem* rings;
         float radius;
+        float geometryScale;
         Vec3f semiAxes;
-        ResourceHandle model;
+        ResourceHandle geometry;
         Quatf orientation;
         std::vector<EclipseShadow>* eclipseShadows;
-        std::vector<Location*>* locations;
     };
 
     class StarVertexBuffer
@@ -284,7 +367,7 @@ class Renderer
         void start();
         void render();
         void finish();
-        void addStar(const Point3f&, const Color&, float);
+        void addStar(const Vec3f&, const Color&, float);
         void setBillboardOrientation(const Quatf&);
 
     private:
@@ -306,13 +389,13 @@ class Renderer
         void startSprites(const GLContext&);
         void render();
         void finish();
-        void addStar(const Point3f& f, const Color&, float);
+        void addStar(const Vec3f& f, const Color&, float);
 		void setTexture(Texture*);
 
     private:
         struct StarVertex
         {
-            Point3f position;
+            Vec3f position;
             float size;
             unsigned char color[4];
             float pad;
@@ -370,12 +453,6 @@ class Renderer
         };
     };
 
-    enum LabelAlignment
-    {
-        AlignCenter,
-        AlignLeft,
-        AlignRight
-    };
     typedef ObjectLabel<Star>          StarLabel;
     typedef ObjectLabel<DeepSkyObject> DSOLabel;    // currently not used
     
@@ -397,13 +474,31 @@ class Renderer
     void renderDeepSkyObjects(const Universe&,
                               const Observer&,
                               float faintestMagNight);
-    void renderCelestialSphere(const Observer& observer);
-    void buildRenderLists(const Star& sun,
-                          const PlanetarySystem* solSystem,
+    void renderSkyGrids(const Observer& observer);
+    void renderSelectionPointer(const Observer& observer,
+                                double now,
+                                const Frustum& viewFrustum,
+                                const Selection& sel);
+
+    void buildRenderLists(const Point3d& astrocentricObserverPos,
+                          const Frustum& viewFrustum,
+                          const Vec3d& viewPlaneNormal,
+                          const Vec3d& frameCenter,
+                          const FrameTree* tree,
                           const Observer& observer,
-                          double now,
-                          std::vector<LightSource>* lightSourceList,
-                          bool showLabels = false);
+                          double now);
+    void buildOrbitLists(const Point3d& astrocentricObserverPos,
+                         const Quatf& observerOrientation,
+                         const Frustum& viewFrustum,
+                         const FrameTree* tree,
+                         double now);
+    void buildLabelLists(const Frustum& viewFrustum,
+                         double now);
+
+    void addRenderListEntries(RenderListEntry& rle,
+                              Body& body,
+                              bool isLabeled);
+
     void addStarOrbitToRenderList(const Star& star,
                                   const Observer& observer,
                                   double now);
@@ -423,7 +518,6 @@ class Renderer
                       float appMag,
                       const Observer& observer,
                       const Quatf& cameraOrientation,
-                      const vector<LightSource>& lightSources,
                       float, float);
 
     void renderStar(const Star& star,
@@ -433,44 +527,26 @@ class Renderer
                     Quatf orientation,
                     double now,
                     float, float);
-
-    void renderAxes(Body& body,
-                    Point3f pos,
-                    float distance,
-                    double now,
-                    float nearPlaneDistance,
-                    float farPlaneDistance,
-                    RenderListEntry::RenderableType renderableType);
-
-    void renderSunDirection(Body& body,
-                            Point3f pos,
-                            float distance,
-                            double now,
-                            const vector<LightSource>& lightSources,
-                            float nearPlaneDistance,
-                            float farPlaneDistance);
-
-    void renderVelocityVector(Body& body,
-                              Point3f pos,
-                              float distance,
-                              double now,
-                              float nearPlaneDistance,
-                              float farPlaneDistance);
+    
+    void renderReferenceMark(const ReferenceMark& refMark,
+                             Point3f pos,
+                             float distance,
+                             double now,
+                             float nearPlaneDistance);
 
     void renderCometTail(const Body& body,
                          Point3f pos,
                          double now,
-                         const vector<LightSource>& lightSources,
                          float discSizeInPixels);
 
-    void renderBodyAsParticle(Point3f center,
-                              float appMag,
-                              float _faintestMag,
-                              float discSizeInPixels,
-                              Color color,
-                              const Quatf& cameraOrientation,
-                              float renderDistance,
-                              bool useHalos);
+    void renderObjectAsPoint_nosprite(Point3f center,
+                                      float radius,
+                                      float appMag,
+                                      float _faintestMag,
+                                      float discSizeInPixels,
+                                      Color color,
+                                      const Quatf& cameraOrientation,
+                                      bool useHalos);
     void renderObjectAsPoint(Point3f center,
                              float radius,
                              float appMag,
@@ -486,15 +562,13 @@ class Renderer
                                    const Quatf& orientation,
                                    Vec3f semiAxes,
                                    const Vec3f& sunDirection,
-                                   Color ambientColor,
+                                   const LightingState& ls,
                                    float fade,
                                    bool lit);
 
-    void renderLocations(const vector<Location*>& locations,
-                         const Quatf& cameraOrientation,
-                         const Point3d& bodyPosition,
-                         const Quatd& bodyOrientation,
-                         float scale);
+    void renderLocations(const Body& body,
+                         const Vec3d& bodyPosition,
+                         const Quatd& bodyOrientation);
                    
     // Render an item from the render list                   
     void renderItem(const RenderListEntry& rle,
@@ -513,14 +587,32 @@ class Renderer
                              const Observer& observer);
     void renderParticles(const std::vector<Particle>& particles,
                          Quatf orientation);
-    void renderLabels(FontStyle fs, LabelAlignment la);
-    std::vector<Label>::iterator renderSortedLabels(std::vector<Label>::iterator,
-                                                    float nearDist,
-                                                    float farDist,
-                                                    FontStyle fs);
+    
+    
+    void addAnnotation(std::vector<Annotation>&,
+                       const MarkerRepresentation*,
+                       const std::string& labelText,
+                       Color color,
+                       const Point3f& position,
+                       LabelAlignment halign = AlignLeft,
+                       LabelVerticalAlignment = VerticalAlignBottom,
+                       float size = 0.0f);
+    void renderAnnotations(const std::vector<Annotation>&, FontStyle fs);
+    void renderBackgroundAnnotations(FontStyle fs);
+    void renderForegroundAnnotations(FontStyle fs);
+    std::vector<Annotation>::iterator renderSortedAnnotations(std::vector<Annotation>::iterator,
+                                                              float nearDist,
+                                                              float farDist,
+                                                              FontStyle fs);
+    std::vector<Renderer::Annotation>::iterator renderAnnotations(std::vector<Annotation>::iterator startIter,
+                                                                  std::vector<Annotation>::iterator endIter,
+                                                                  float nearDist,
+                                                                  float farDist,
+                                                                  FontStyle fs);
+
     void renderMarkers(const MarkerList&,
-                       const UniversalCoord& position,
-                       const Quatf& orientation,
+                       const UniversalCoord& cameraPosition,
+                       const Quatd& cameraOrientation,
                        double jd);
 
     void renderOrbit(const OrbitPathListEntry&,
@@ -530,12 +622,46 @@ class Renderer
                      float nearDist,
                      float farDist);
 
+#ifdef USE_HDR
+ private:
+    int sceneTexWidth, sceneTexHeight;
+    GLfloat sceneTexWScale, sceneTexHScale;
+    GLsizei blurBaseWidth, blurBaseHeight;
+    GLuint sceneTexture;
+    Texture **blurTextures;
+    Texture *blurTempTexture;
+    GLuint gaussianLists[4];
+    GLint blurFormat;
+    bool useBlendSubtract;
+    bool useLuminanceAlpha;
+    bool bloomEnabled;
+    float maxBodyMag;
+    float exposure, exposurePrev;
+    float brightPlus;
+
+    void genBlurTexture(int blurLevel);
+    void genBlurTextures();
+    void genSceneTexture();
+    void renderToBlurTexture(int blurLevel);
+    void renderToTexture(const Observer& observer,
+                         const Universe& universe,
+                         float faintestMagNight,
+                         const Selection& sel);
+    void drawSceneTexture();
+    void drawBlur();
+    void drawGaussian3x3(float xdelta, float ydelta, GLsizei width, GLsizei height, float blend);
+    void drawGaussian5x5(float xdelta, float ydelta, GLsizei width, GLsizei height, float blend);
+    void drawGaussian9x9(float xdelta, float ydelta, GLsizei width, GLsizei height, float blend);
+    void drawBlendedVertices(float xdelta, float ydelta, float blend);
+#endif
+
  private:
     GLContext* context;
 
     int windowWidth;
     int windowHeight;
     float fov;
+    double cosViewConeAngle;
     int screenDpi;
     float corrFac;
     float pixelSize;
@@ -561,19 +687,23 @@ class Renderer
     Color ambientColor;
     std::string displayedSurface;
 
+    Quatf m_cameraOrientation;
     StarVertexBuffer* starVertexBuffer;
     PointStarVertexBuffer* pointStarVertexBuffer;
 	PointStarVertexBuffer* glareVertexBuffer;
     std::vector<RenderListEntry> renderList;
+    std::vector<SecondaryIlluminator> secondaryIlluminators;
     std::vector<DepthBufferPartition> depthPartitions;
     std::vector<Particle> glareParticles;
-    std::vector<Label> labels;
-    std::vector<Label> depthSortedLabels;
+    std::vector<Annotation> backgroundAnnotations;
+    std::vector<Annotation> foregroundAnnotations;
+    std::vector<Annotation> depthSortedAnnotations;
+    std::vector<Annotation> objectAnnotations;
     std::vector<OrbitPathListEntry> orbitPathList;
     std::vector<EclipseShadow> eclipseShadows[MaxLights];
     std::vector<const Star*> nearStars;
 
-    std::list<std::vector<LightSource>* > lightSourceLists;
+    std::vector<LightSource> lightSourceList;
 
     double modelMatrix[16];
     double projMatrix[16];
@@ -584,13 +714,15 @@ class Renderer
     bool usePointSprite;
     bool useClampToBorder;
     unsigned int textureResolution;
-
     DetailOptions detailOptions;
     
     bool useNewStarRendering;
 
     uint32 frameCount;
 
+    int currentIntervalIndex;
+
+
  public:
     struct OrbitSample 
     {
@@ -609,14 +741,13 @@ class Renderer
     
     struct CachedOrbit
     {
-        Body* body;
         std::vector<OrbitSample> trajectory;
         std::vector<OrbitSection> sections;
         uint32 lastUsed;
     };
 
  private:
-    typedef std::map<Body*, CachedOrbit*> OrbitCache;
+    typedef std::map<const Orbit*, CachedOrbit*> OrbitCache;
     OrbitCache orbitCache;
     uint32 lastOrbitCacheFlush;
 
@@ -636,26 +767,52 @@ class Renderer
     bool videoSync;
     bool settingsChanged;
 
+    // True if we're in between a begin/endObjectAnnotations
+    bool objectAnnotationSetOpen;
+
+    double realTime;
+    
+    // Location markers
+ public:
+    MarkerRepresentation mountainRep;
+    MarkerRepresentation craterRep;
+    MarkerRepresentation observatoryRep;
+    MarkerRepresentation cityRep;
+    MarkerRepresentation genericLocationRep;
+    MarkerRepresentation galaxyRep;
+    MarkerRepresentation nebulaRep;
+    MarkerRepresentation openClusterRep;
+    MarkerRepresentation globularRep;
+
     std::list<RendererWatcher*> watchers;
 
  public:
     // Colors for all lines and labels
     static Color StarLabelColor;
     static Color PlanetLabelColor;
+    static Color DwarfPlanetLabelColor;
     static Color MoonLabelColor;
+    static Color MinorMoonLabelColor;
     static Color AsteroidLabelColor;
     static Color CometLabelColor;
     static Color SpacecraftLabelColor;
     static Color LocationLabelColor;
     static Color GalaxyLabelColor;
-    static Color NebulaLabelColor;
+    static Color GlobularLabelColor;	    
+	static Color NebulaLabelColor;
     static Color OpenClusterLabelColor;
     static Color ConstellationLabelColor;
     static Color EquatorialGridLabelColor;
+    static Color PlanetographicGridLabelColor;
+    static Color GalacticGridLabelColor;
+    static Color EclipticGridLabelColor;
+    static Color HorizonGridLabelColor;
 
     static Color StarOrbitColor;
     static Color PlanetOrbitColor;
+    static Color DwarfPlanetOrbitColor;
     static Color MoonOrbitColor;
+    static Color MinorMoonOrbitColor;
     static Color AsteroidOrbitColor;
     static Color CometOrbitColor;
     static Color SpacecraftOrbitColor;
@@ -664,6 +821,14 @@ class Renderer
     static Color ConstellationColor;
     static Color BoundaryColor;
     static Color EquatorialGridColor;
+    static Color PlanetographicGridColor;
+    static Color PlanetEquatorColor;
+    static Color GalacticGridColor;
+    static Color EclipticGridColor;
+    static Color HorizonGridColor;
+    static Color EclipticColor;
+
+    static Color SelectionCursorColor;
 };
 
 
