@@ -1,6 +1,6 @@
 // celestiacore.h
 // 
-// Copyright (C) 2001, Chris Laurel <claurel@shatters.net>
+// Copyright (C) 2001-2009, the Celestia Development Team
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -35,7 +35,7 @@ class Url;
 // class CelestiaWatcher;
 class CelestiaCore;
 
-class astro::Date;
+// class astro::Date;
 
 typedef Watcher<CelestiaCore> CelestiaWatcher;
 
@@ -296,6 +296,7 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void splitView(View::Type type, View* av = NULL, float splitPos = 0.5f);
     void singleView(View* av = NULL);
     void deleteView(View* v = NULL);
+    void setActiveView(View* v = NULL);
     bool getFramesVisible() const;
     void setFramesVisible(bool);
     bool getActiveFrameVisible() const;
@@ -338,6 +339,9 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     
     void setCursorHandler(CursorHandler*);
     CursorHandler* getCursorHandler() const;
+
+    void toggleReferenceMark(const std::string& refMark, Selection sel = Selection());
+    bool referenceMarkEnabled(const std::string& refMark, Selection sel = Selection()) const;
     
  private:
     bool readStars(const CelestiaConfig&, ProgressNotifier*);
@@ -448,8 +452,8 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     std::vector<Url>::size_type historyCurrent;
     std::string startURL;
 
-    std::vector<View*> views;
-    int activeView;
+    std::list<View*> views;
+    std::list<View*>::iterator activeView;
     bool showActiveViewFrame;
     bool showViewFrames;
     View *resizeSplit;
