@@ -10,7 +10,7 @@
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
-#include <string>
+#include <cstring>
 #include <sstream>
 #include <algorithm>
 #include <set>
@@ -49,7 +49,7 @@ bool EclipseFinder::testEclipse(const Body& receiver, const Body& caster,
     // ignore eclipses where the caster is not an ellipsoid, since we can't
     // generate correct shadows in this case.
     if (caster.getRadius() >= receiver.getRadius() * MinRelativeOccluderRadius &&
-        caster.getModel() == InvalidResource)
+        caster.isEllipsoid())
     {
         // All of the eclipse related code assumes that both the caster
         // and receiver are spherical.  Irregular receivers will work more
@@ -59,8 +59,8 @@ bool EclipseFinder::testEclipse(const Body& receiver, const Body& caster,
         // less than the distance between the sun and the receiver.  This
         // approximation works everywhere in the solar system, and likely
         // works for any orbitally stable pair of objects orbiting a star.
-        Point3d posReceiver = receiver.getHeliocentricPosition(now);
-        Point3d posCaster = caster.getHeliocentricPosition(now);
+        Point3d posReceiver = receiver.getAstrocentricPosition(now);
+        Point3d posCaster = caster.getAstrocentricPosition(now);
 
         const Star* sun = receiver.getSystem()->getStar();
         assert(sun != NULL);
