@@ -432,6 +432,12 @@ static void dirFixup(char *argv0) {
 
 int main(int argc, char* argv[])
 {
+	setlocale(LC_ALL, "");
+	setlocale(LC_NUMERIC, "C");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset(PACKAGE, "UTF-8");
+	textdomain(PACKAGE);
+			
    #ifdef MACOSX
    #define BUNDLEONLY 1
    #ifndef BUNDLEONLY
@@ -456,11 +462,12 @@ int main(int argc, char* argv[])
     ready = false;
 
     char c;
-    while ((c = getopt(argc, argv, "v::")) > -1)
+	int startfile = 0;
+    while ((c = getopt(argc, argv, "v::f")) > -1)
     {
         if (c == '?')
         {
-            cout << "Usage: celestia [-v]\n";
+            cout << "Usage: celestia [-v] [-f <filename>]\n";
             exit(1);
         }
         else if (c == 'v')
@@ -470,6 +477,9 @@ int main(int argc, char* argv[])
             else
                 SetDebugVerbosity(0);
         }
+		else if (c == 'f') {
+			startfile = 1;
+		}
     }
 
     appCore = new CelestiaCore();
@@ -525,6 +535,16 @@ int main(int argc, char* argv[])
     appCore->setTimeZoneName(tzname[daylight?0:1]);
     #endif
 
+	if (startfile == 1) {
+		if (argv[argc - 1][0] == '-') {
+			cout << "Missing Filename.\n";
+			return 1;
+		}
+	
+		cout << "*** Using CEL File: " << argv[argc - 1] << endl;
+		appCore->runScript(argv[argc - 1]);
+	}
+
     ready = true;
     glutMainLoop();
 
