@@ -69,7 +69,9 @@ static void setLabelMode(AppData* a, int mode, gboolean state);
 void actionCopyURL(GtkAction*, AppData* app)
 {
 	GtkClipboard* cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-	Url url(app->core);
+	CelestiaState appState;
+	appState.captureState(app->core);
+	Url url(appState, Url::CurrentVersion);
 	gtk_clipboard_set_text(cb, url.getAsString().c_str(), -1);
 }
 
@@ -723,7 +725,7 @@ void actionHelpAbout(GtkAction*, AppData* app)
 	gtk_show_about_dialog(GTK_WINDOW(app->mainWindow),
 	                     "name", "Celestia",
 	                     "version", VERSION,
-	                     "copyright", "Copyright \xc2\xa9 2001-2008 Celestia Development Team",
+	                     "copyright", "Copyright \xc2\xa9 2001-2009 Celestia Development Team",
 	                     "comments", FRONTEND " Front-End",
 	                     "website", "http://celestia.sf.net",
 	                     "authors", authors,
@@ -815,18 +817,42 @@ void actionRenderConstellations(GtkToggleAction* action, AppData* app)
 }
 
 
+void actionRenderEclipticGrid(GtkToggleAction* action, AppData* app)
+{
+	setRenderFlag(app, Renderer::ShowEclipticGrid, gtk_toggle_action_get_active(action));
+}
+
+
 void actionRenderEclipseShadows(GtkToggleAction* action, AppData* app)
 {
 	setRenderFlag(app, Renderer::ShowEclipseShadows, gtk_toggle_action_get_active(action));
 }
 
 
+void actionRenderGalacticGrid(GtkToggleAction* action, AppData* app)
+{
+	setRenderFlag(app, Renderer::ShowGalacticGrid, gtk_toggle_action_get_active(action));
+}
+
+
 void actionRenderGalaxies(GtkToggleAction* action, AppData* app)
 {
 	setRenderFlag(app, Renderer::ShowGalaxies, gtk_toggle_action_get_active(action));
 }
 
 
+void actionRenderGlobulars(GtkToggleAction* action, AppData* app)
+{
+	setRenderFlag(app, Renderer::ShowGlobulars, gtk_toggle_action_get_active(action));
+}
+
+
+void actionRenderHorizontalGrid(GtkToggleAction* action, AppData* app)
+{
+	setRenderFlag(app, Renderer::ShowHorizonGrid, gtk_toggle_action_get_active(action));
+}
+
+
 void actionRenderMarkers(GtkToggleAction* action, AppData* app)
 {
 	setRenderFlag(app, Renderer::ShowMarkers, gtk_toggle_action_get_active(action));
@@ -929,6 +955,12 @@ void actionLabelGalaxies(GtkToggleAction* action, AppData* app)
 }
 
 
+void actionLabelGlobulars(GtkToggleAction* action, AppData* app)
+{
+	setLabelMode(app, Renderer::GlobularLabels, gtk_toggle_action_get_active(action));
+}
+
+
 void actionLabelLocations(GtkToggleAction* action, AppData* app)
 {
 	setLabelMode(app, Renderer::LocationLabels, gtk_toggle_action_get_active(action));
@@ -1219,7 +1251,7 @@ void resyncLabelActions(AppData* app)
 	/* Simply for readability */
 	int f = app->renderer->getLabelMode();
 	
-	for (int i = Renderer::StarLabels; i <= Renderer::I18nConstellationLabels; i *= 2)
+	for (int i = Renderer::StarLabels; i <= Renderer::GlobularLabels; i *= 2)
 	{
 		switch (i)
 		{
@@ -1234,6 +1266,7 @@ void resyncLabelActions(AppData* app)
 			case Renderer::CometLabels: actionName = "LabelComets"; break;
 			case Renderer::NebulaLabels: actionName = "LabelNebulae"; break;
 			case Renderer::OpenClusterLabels: actionName = "LabelOpenClusters"; break;
+			case Renderer::GlobularLabels: actionName = "LabelGlobulars"; break;
 			case Renderer::I18nConstellationLabels: /* Not used yet */
 			default: actionName = NULL;
 		}
@@ -1261,9 +1294,9 @@ void resyncRenderActions(AppData* app)
 	
 	/* Unlike the other interfaces, which go through each menu item and set
 	 * the corresponding renderFlag, we go the other way and set the menu
-	 * based on the renderFlag. Last one is ShowOpenClusters. */
+	 * based on the renderFlag. Last one is ShowEcliptic. */
 	
-	for (int i = Renderer::ShowStars; i <= Renderer::ShowOpenClusters; i *= 2)
+	for (int i = Renderer::ShowStars; i <= Renderer::ShowEcliptic; i *= 2)
 	{
 		switch (i)
 		{
@@ -1287,6 +1320,12 @@ void resyncRenderActions(AppData* app)
 			case Renderer::ShowPartialTrajectories: actionName = NULL; break; /* Not useful yet */
 			case Renderer::ShowNebulae: actionName = "RenderNebulae"; break;
 			case Renderer::ShowOpenClusters: actionName = "RenderOpenClusters"; break;
+			case Renderer::ShowGlobulars: actionName = "RenderGlobulars"; break;
+			case Renderer::ShowCloudShadows: actionName = NULL; break; /* Not implemented yet */
+			case Renderer::ShowGalacticGrid: actionName = "RenderGalacticGrid"; break;
+			case Renderer::ShowEclipticGrid: actionName = "RenderEclipticGrid"; break;
+			case Renderer::ShowHorizonGrid: actionName = "RenderHorizontalGrid"; break;
+			case Renderer::ShowEcliptic: actionName = NULL; break; /* Not implemented yet */
 			default: actionName = NULL;
 		}
 		
