@@ -32,12 +32,14 @@
 #include <qspinbox.h>
 #include <qlabel.h>
 #include <qlayout.h>
-
+#include <qlineedit.h>
+#include <qvalidator.h>
 
 #include "kdeapp.h"
 #include "kdepreferencesdialog.h"
 #include "celengine/render.h"
 #include "celengine/glcontext.h"
+#include "celengine/astro.h"
 
 static uint32 FilterOtherLocations = ~(Location::City |
                     Location::Observatory |
@@ -55,9 +57,9 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
 
     setCaption(i18n("Celestia Preferences"));
     appCore=core;
-    
+
     this->parent = (KdeApp*)parent;
-  
+
     // Render page
     QGrid* renderFrame = addGridPage(2, Qt::Horizontal, i18n("Rendering"), i18n("Rendering"),
         KGlobal::iconLoader()->loadIcon("configure", KIcon::NoGroup));
@@ -101,6 +103,10 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     actionColl->action("showCloudMaps")->connect(showCloudMapsCheck, SIGNAL(clicked()), SLOT(activate()));
     showCloudMapsCheck->setChecked(renderFlags & Renderer::ShowCloudMaps);
 
+    QCheckBox* showCloudShadowsCheck = new QCheckBox(i18n("Cloud Shadows"), showGroup);
+    actionColl->action("showCloudShadows")->connect(showCloudShadowsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showCloudShadowsCheck->setChecked(renderFlags & Renderer::ShowCloudShadows);
+
     QCheckBox* showNightMapsCheck = new QCheckBox(i18n("Night Side Lights"), showGroup);
     actionColl->action("showNightMaps")->connect(showNightMapsCheck, SIGNAL(clicked()), SLOT(activate()));
     showNightMapsCheck->setChecked(renderFlags & Renderer::ShowNightMaps);
@@ -161,6 +167,10 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     QLabel* labelsLabel = new QLabel(i18n("Labels"), labelGroup);
     labelGroupLayout->addWidget(labelsLabel, 0, 1);
 
+    QCheckBox* showStarOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showStarOrbits")->connect(showStarOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showStarOrbitsCheck->setChecked(orbitMask & Body::Stellar);
+    labelGroupLayout->addWidget(showStarOrbitsCheck, 1, 0, Qt::AlignHCenter);
     QCheckBox* showStarLabelsCheck = new QCheckBox(i18n("Stars"), labelGroup);
     actionColl->action("showStarLabels")->connect(showStarLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showStarLabelsCheck->setChecked(labelMode & Renderer::StarLabels);
@@ -276,6 +286,16 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     infoTextCombo->setCurrentItem(savedHudDetail);
     ((KdeApp*)parent)->connect(infoTextCombo, SIGNAL(activated(int)), SLOT(slotHudDetail(int)));
 
+    QGroupBox* textureGroup = new QGroupBox(1, Qt::Vertical, i18n("Textures"), vbox1);
+    new QLabel(i18n("Resolution: "), textureGroup);
+    QComboBox* textureResCombo = new QComboBox(textureGroup);
+    textureResCombo->insertItem(i18n("Low"));
+    textureResCombo->insertItem(i18n("Medium"));
+    textureResCombo->insertItem(i18n("High"));
+    savedTextureRes = appCore->getRenderer()->getResolution();
+    textureResCombo->setCurrentItem(savedTextureRes);
+    connect(textureResCombo, SIGNAL(activated(int)), SLOT(slotTextureRes(int)));
+
     QGroupBox* fovGroup = new QGroupBox(2, Qt::Horizontal, i18n("Automatic FOV"), vbox1);
     new QLabel(i18n("Screen DPI: "), fovGroup);
     new QLabel(QString::number(appCore->getScreenDpi(), 10), fovGroup);
@@ -349,17 +369,28 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
         KGlobal::iconLoader()->loadIcon("clock", KIcon::NoGroup));
 
     savedDisplayLocalTime = appCore->getTimeZoneBias();
-    QGroupBox* displayTimezoneGroup = new QGroupBox(1, Qt::Vertical, i18n("Display"), timeFrame);
-    new QLabel(i18n("Timezone: "), displayTimezoneGroup);
-    displayTimezoneCombo = new QComboBox(displayTimezoneGroup);
+    QGroupBox* displayTimezoneGroup = new QGroupBox(1, Qt::Horizontal, i18n("Display"), timeFrame);
+    QHBox *hbox0 = new QHBox(displayTimezoneGroup);
+    new QLabel(i18n("Timezone: "), hbox0);
+    displayTimezoneCombo = new QComboBox(hbox0);
     displayTimezoneCombo->insertItem(i18n("UTC"));
     displayTimezoneCombo->insertItem(i18n("Local Time"));
     displayTimezoneCombo->setCurrentItem((appCore->getTimeZoneBias()==0)?0:1);
     ((KdeApp*)parent)->connect(displayTimezoneCombo, SIGNAL(activated(int)), SLOT(slotDisplayLocalTime()));
     displayTimezoneGroup->addSpace(0);
 
+    QHBox *hbox1 = new QHBox(displayTimezoneGroup);
+    new QLabel(i18n("Format: "), hbox1);
+    QComboBox *timeFormatCombo = new QComboBox(hbox1);
+    timeFormatCombo->insertItem(i18n("Local Format"));
+    timeFormatCombo->insertItem("YYYY MMM DD HH:MM:SS TZ");
+    timeFormatCombo->insertItem("YYYY MMM DD HH:MM:SS Offset");
+    savedDateFormat = appCore->getDateFormat();
+    timeFormatCombo->setCurrentItem((int)savedDateFormat);
+    connect(timeFormatCombo, SIGNAL(activated(int)), SLOT(slotDateFormat(int)));
+
     QGroupBox* setTimezoneGroup = new QGroupBox(1, Qt::Horizontal, i18n("Set"), timeFrame);
-    new QLabel(i18n("Local Time is only supported for dates between 1902 and 2037.\n"), setTimezoneGroup);   
+    new QLabel(i18n("Local Time is only supported for dates between 1902 and 2037.\n"), setTimezoneGroup);
     QHBox *hbox2 = new QHBox(setTimezoneGroup);
     new QLabel(i18n("Timezone: "), hbox2);
     setTimezoneCombo = new QComboBox(hbox2);
@@ -372,7 +403,7 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     KGlobal::config()->setGroup(0);
     connect(setTimezoneCombo, SIGNAL(activated(int)), SLOT(slotTimeHasChanged()));
 
-        
+
     QHBox *hboxdate = new QHBox(setTimezoneGroup);
     QLabel* spacerdate1 = new QLabel(" ", hboxdate);
     timeFrame->setStretchFactor(spacerdate1, 2);
@@ -384,7 +415,7 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     DSpin->setWrapping(true);
     QLabel* spacerdate2 = new QLabel(" ", hboxdate);
     timeFrame->setStretchFactor(spacerdate2, 2);
-  
+
     QVBox *vbox3 = new QVBox(setTimezoneGroup);
     QHBox *hbox3 = new QHBox(vbox3);
     QLabel* spacer1 = new QLabel(" ", hbox3);
@@ -403,7 +434,17 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     QLabel* spacer3 = new QLabel(" ", timeFrame);
     timeFrame->setStretchFactor(spacer3, 2);
 
-    setTime(appCore->getSimulation()->getTime());
+    QHBox *hbox4 = new QHBox(setTimezoneGroup);
+    new QLabel(i18n("Julian Date: "), hbox4);
+    new QLabel(" ", hbox4);
+    julianDateEdit = new QLineEdit(hbox4);
+    QDoubleValidator *julianDateValidator = new QDoubleValidator(julianDateEdit);
+    julianDateEdit->setValidator(julianDateValidator);
+    julianDateEdit->setAlignment(Qt::AlignRight);
+    connect(julianDateEdit, SIGNAL(lostFocus()), SLOT(slotJDHasChanged()));
+
+
+    setTime(astro::TDBtoUTC(appCore->getSimulation()->getTime()));
     connect(YSpin, SIGNAL(valueChanged(int)), SLOT(slotTimeHasChanged()));
     connect(MSpin, SIGNAL(valueChanged(int)), SLOT(slotTimeHasChanged()));
     connect(DSpin, SIGNAL(valueChanged(int)), SLOT(slotTimeHasChanged()));
@@ -423,7 +464,7 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
         QHBox* ltBox = new QHBox(setTimezoneGroup);
         char time[50];
         sel_name = selection.body()->getName();
-        Vec3d v = selection.getPosition(appCore->getSimulation()->getTime()) - 
+        Vec3d v = selection.getPosition(appCore->getSimulation()->getTime()) -
                   appCore->getSimulation()->getObserver().getPosition();
         double dist = astro::lightYearsToKilometers(v.length()*1e-6);
         double lt = dist / astro::speedOfLight;
@@ -432,12 +473,12 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
         double lt_s = (lt - lt_h * 3600 - lt_m * 60);
         if (lt_h == 0) snprintf(time, 50, "%d min %02.1f s", lt_m, lt_s);
         else snprintf(time, 50, "%ld h %02d min %02.1f s", lt_h, lt_m, lt_s);
-        
-        new QLabel(i18n("\nSelection: " + QString(sel_name.c_str()) 
+
+        new QLabel(i18n("\nSelection: " + QString(sel_name.c_str())
         + QString("\nLight Travel Time: %2").arg(time)), ltBox);
-        
+
         KPushButton *ltButton = new KPushButton(ltBox);
-        ltButton->setToggleButton(true);        
+        ltButton->setToggleButton(true);
 
         if (!appCore->getLightDelayActive())
             ltButton->setText(i18n("Include Light Travel Time"));
@@ -452,6 +493,7 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     QVBox* openGL = addVBoxPage(i18n("OpenGL"), i18n("OpenGL"),
         KGlobal::iconLoader()->loadIcon("misc", KIcon::NoGroup));
 
+
     renderPathCombo = new QComboBox(openGL);
     savedRenderPath = (int)appCore->getRenderer()->getGLContext()->getRenderPath();
     if (appCore->getRenderer()->getGLContext()->renderPathSupported(GLContext::GLPath_Basic))
@@ -490,9 +532,16 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     renderPathLabel->setTextFormat(Qt::RichText);
     setRenderPathLabel();
 
+    QCheckBox* videoSyncCheck = new QCheckBox(i18n("Sync framerate to video refresh rate"), openGL);
+    actionColl->action("toggleVideoSync")->connect(videoSyncCheck, SIGNAL(clicked()), SLOT(activate()));
+    savedVideoSync = appCore->getRenderer()->getVideoSync();
+    videoSyncCheck->setChecked(savedVideoSync);
+
+
     QTextEdit* edit = new QTextEdit(openGL);
     edit->append(((KdeApp*)parent)->getOpenGLInfo());
     edit->setFocusPolicy(QWidget::NoFocus);
+    edit->setCursorPosition(0, 0);
 
 
     // Key bindings page
@@ -506,20 +555,20 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
 
 KdePreferencesDialog::~KdePreferencesDialog() {
 }
-  
+
 
 void KdePreferencesDialog::setNow() {
     time_t date_t;
     time(&date_t);
     struct tm* tm;
-    if (setTimezoneCombo->currentItem() != 0) { 
-        tm = localtime ( &date_t); 
+    if (setTimezoneCombo->currentItem() != 0) {
+        tm = localtime ( &date_t);
     } else {
-        tm = gmtime ( &date_t);   
+        tm = gmtime ( &date_t);
     }
     YSpin->setValue(tm->tm_year + 1900);
     MSpin->setValue(tm->tm_mon + 1);
-    DSpin->setValue(tm->tm_mday);    
+    DSpin->setValue(tm->tm_mday);
     hSpin->setValue(tm->tm_hour);
     mSpin->setValue(tm->tm_min);
     sSpin->setValue(tm->tm_sec);
@@ -527,9 +576,9 @@ void KdePreferencesDialog::setNow() {
 
 void KdePreferencesDialog::ltSubstract() {
     double d;
-    
+
     d = getTime();
-    
+
     Selection selection = appCore->getSimulation()->getSelection();
     std::string sel_name;
 
@@ -553,36 +602,39 @@ void KdePreferencesDialog::ltSubstract() {
 }
 
 void KdePreferencesDialog::setTime(double d) {
-    if (setTimezoneCombo->currentItem() != 0 && d < 2465442 && d > 2415733) { 
+    if (setTimezoneCombo->currentItem() != 0 && d < 2465442 && d > 2415733) {
         time_t date_t = (int) astro::julianDateToSeconds( d - (float)astro::Date(1970, 1, 1) );
 
         struct tm* tm;
-        tm = localtime ( &date_t); 
+        tm = localtime ( &date_t);
         YSpin->setValue(tm->tm_year + 1900);
         MSpin->setValue(tm->tm_mon + 1);
-        DSpin->setValue(tm->tm_mday);    
+        DSpin->setValue(tm->tm_mday);
         hSpin->setValue(tm->tm_hour);
         mSpin->setValue(tm->tm_min);
-        sSpin->setValue(tm->tm_sec);    
+        sSpin->setValue(tm->tm_sec);
     } else {
         astro::Date date(d);
         YSpin->setValue(date.year);
         MSpin->setValue(date.month);
-        DSpin->setValue(date.day);    
+        DSpin->setValue(date.day);
         hSpin->setValue(date.hour);
         mSpin->setValue(date.minute);
         sSpin->setValue(int(date.seconds));
     }
+
+    QString jd;
+    julianDateEdit->setText(jd.setNum(d, 'f'));
 }
 
 double KdePreferencesDialog::getTime() const {
     double d;
-    
-    if (setTimezoneCombo->currentItem() == 0 || YSpin->value() < 1902 || YSpin->value() > 2037) {  
+
+    if (setTimezoneCombo->currentItem() == 0 || YSpin->value() < 1902 || YSpin->value() > 2037) {
         astro::Date date(0.0);
         date.year=YSpin->value();
         date.month=MSpin->value();
-        date.day=DSpin->value();        
+        date.day=DSpin->value();
         date.hour=hSpin->value();
         date.minute=mSpin->value();
         date.seconds=float(sSpin->value());
@@ -597,6 +649,10 @@ double KdePreferencesDialog::getTime() const {
         time.tm_sec = sSpin->value();
         d = astro::secondsToJulianDate(mktime(&time)) + (double) astro::Date(1970, 1, 1);
     }
+
+    QString jd;
+    julianDateEdit->setText(jd.setNum(d, 'f'));
+
     return d;
 }
 
@@ -616,6 +672,9 @@ void KdePreferencesDialog::slotCancel() {
     appCore->setDistanceToScreen(savedDistanceToScreen);
     appCore->getSimulation()->getActiveObserver()->setLocationFilter(savedLocationFilter);
     appCore->getRenderer()->setMinimumFeatureSize(savedMinFeatureSize);
+    appCore->getRenderer()->setVideoSync(savedVideoSync);
+    appCore->getRenderer()->setResolution(savedTextureRes);
+    appCore->setDateFormat(savedDateFormat);
     reject();
 }
 
@@ -629,7 +688,10 @@ void KdePreferencesDialog::slotApply() {
     savedRenderPath = (int)appCore->getRenderer()->getGLContext()->getRenderPath();
     savedDistanceToScreen = appCore->getDistanceToScreen();
     savedLocationFilter = appCore->getSimulation()->getActiveObserver()->getLocationFilter();
-    savedMinFeatureSize = appCore->getRenderer()->getMinimumFeatureSize();
+    savedMinFeatureSize = (int)appCore->getRenderer()->getMinimumFeatureSize();
+    savedVideoSync = appCore->getRenderer()->getVideoSync();
+    savedTextureRes = appCore->getRenderer()->getResolution();
+    savedDateFormat = appCore->getDateFormat();
 
     keyChooser->commitChanges();
 
@@ -638,15 +700,15 @@ void KdePreferencesDialog::slotApply() {
     KGlobal::config()->setGroup(0);
 
     if (timeHasChanged) {
-      if (setTimezoneCombo->currentItem() == 0 || YSpin->value() < 1902 || YSpin->value() > 2037) {  
+      if (setTimezoneCombo->currentItem() == 0 || YSpin->value() < 1902 || YSpin->value() > 2037) {
         astro::Date date(0.0);
         date.year=YSpin->value();
         date.month=MSpin->value();
-        date.day=DSpin->value();        
+        date.day=DSpin->value();
         date.hour=hSpin->value();
         date.minute=mSpin->value();
         date.seconds=float(sSpin->value());
-        appCore->getSimulation()->setTime((double) date);
+        appCore->getSimulation()->setTime(astro::UTCtoTDB((double) date));
         appCore->getSimulation()->update(0.0);
       } else {
         struct tm time;
@@ -656,19 +718,24 @@ void KdePreferencesDialog::slotApply() {
         time.tm_hour = hSpin->value();
         time.tm_min = mSpin->value();
         time.tm_sec = sSpin->value();
-        appCore->getSimulation()->setTime(astro::secondsToJulianDate(mktime(&time)) + (double) astro::Date(1970, 1, 1));
-        appCore->getSimulation()->update(0.0);       
+        appCore->getSimulation()->setTime(astro::UTCtoTDB(astro::secondsToJulianDate(mktime(&time)) + (double) astro::Date(1970, 1, 1)));
+        appCore->getSimulation()->update(0.0);
       }
     }
 }
 
 void KdePreferencesDialog::slotTimeHasChanged() {
     timeHasChanged = true;
+    getTime();
+}
+
+void KdePreferencesDialog::slotJDHasChanged() {
+    setTime(julianDateEdit->text().toDouble());
 }
 
 void KdePreferencesDialog::slotFaintestVisible(int m) {
     char buff[20];
-    
+
     parent->slotFaintestVisible(m / 100.);
     sprintf(buff, "%.2f", m / 100.);
     faintestLabel->setText(buff);
@@ -684,7 +751,7 @@ void KdePreferencesDialog::slotMinFeatureSize(int s) {
 
 void KdePreferencesDialog::slotAmbientLightLevel(int l) {
     char buff[20];
-    
+
     parent->slotAmbientLightLevel(l / 100.);
     sprintf(buff, "%.2f", l / 100.);
     ambientLabel->setText(buff);
@@ -707,6 +774,14 @@ void KdePreferencesDialog::slotDistanceToScreen(int dts) {
     appCore->setDistanceToScreen(dts * 10);
 }
 
+void KdePreferencesDialog::slotTextureRes(int res) {
+    appCore->getRenderer()->setResolution(res);
+}
+
+void KdePreferencesDialog::slotDateFormat(int format) {
+    appCore->setDateFormat((astro::Date::Format)format);
+}
+
 void KdePreferencesDialog::setRenderPathLabel() {
     switch(appCore->getRenderer()->getGLContext()->getRenderPath()) {
     case GLContext::GLPath_Basic:
