@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 
+#include <libintl.h>
 #include <sstream>
 #include <fstream>
 #include <algorithm>
@@ -29,6 +30,10 @@
 #include <krun.h>
 
 #include "celutil/utf8.h"
+#include "celestia/celestiacore.h"
+#include "celengine/axisarrow.h"
+#include "celengine/planetgrid.h"
+
 
 SelectionPopup::SelectionPopup(QWidget* parent, CelestiaCore* _appCore, Selection _sel):
 KPopupMenu(parent),
@@ -128,8 +133,8 @@ void SelectionPopup::init()
             for (std::vector<Marker>::iterator i = markers->begin(); i < markers->end(); i++)
             {
                 KPopupMenu *objMenu = new KPopupMenu(this);
-                insert(objMenu, (*i).getObject(), false);
-                markMenu->insertItem(QString::fromUtf8(getSelectionName((*i).getObject())), objMenu);
+                insert(objMenu, (*i).object(), false);
+                markMenu->insertItem(QString::fromUtf8(getSelectionName((*i).object())), objMenu);
                 j++;
             }
             insertItem(i18n("Marked objects"), markMenu);
@@ -214,42 +219,47 @@ void SelectionPopup::process(int id)
     {
         if (sim->getUniverse() != NULL)
         {
-            sim->getUniverse()->markObject(sel,
-                                           10.0f,
-                                           Color(0.0f, 1.0f, 0.0f, 0.9f),
-                                           (Marker::Symbol)(actionId - 10),
-                                           1,
-                                           "");
+            MarkerRepresentation markerRep((MarkerRepresentation::Symbol) (actionId - 1), 10.0f, Color(0.0f, 1.0f, 0.0f, 0.9f));
+
+            sim->getUniverse()->markObject(sel, markerRep, 1);
         }
         return;
     }
-    if (actionId >=25 && actionId < 30 && sel.body() != NULL) 
+    if (actionId >=25 && actionId < 31 && sel.body() != NULL) 
     {
+
         switch(actionId)
         {
         case 25:
-            sel.body()->setVisibleReferenceMarks(sel.body()->getVisibleReferenceMarks() ^ Body::BodyAxes);
+		appCore->toggleReferenceMark("body axes");
             break;
         case 26:
-            sel.body()->setVisibleReferenceMarks(sel.body()->getVisibleReferenceMarks() ^ Body::FrameAxes);
+		appCore->toggleReferenceMark("frame axes");
             break;
         case 27:
-            sel.body()->setVisibleReferenceMarks(sel.body()->getVisibleReferenceMarks() ^ Body::SunDirection);
+		appCore->toggleReferenceMark("sun direction");
             break;
         case 28:
-            sel.body()->setVisibleReferenceMarks(sel.body()->getVisibleReferenceMarks() ^ Body::VelocityVector);
+		appCore->toggleReferenceMark("velocity vector");
             break;
+	case 29:
+		appCore->toggleReferenceMark("planetographic grid");
+	    break;
+	case 30:
+		appCore->toggleReferenceMark("terminator");
+	    break;
         }
+
     }
-    if (actionId == 30) {
+    if (actionId == 31) {
         sim->getActiveObserver()->setDisplayedSurface("");
         return;
     }
-    if (actionId > 30) {
+    if (actionId > 31) {
         std::vector<std::string>* altSurfaces = sel.body()->getAlternateSurfaceNames();
-        if (altSurfaces != NULL && (int) altSurfaces->size() > actionId - 31)
+        if (altSurfaces != NULL && (int) altSurfaces->size() > actionId - 32)
         {
-            sim->getActiveObserver()->setDisplayedSurface((*altSurfaces)[actionId - 31]);
+            sim->getActiveObserver()->setDisplayedSurface((*altSurfaces)[actionId - 32]);
         }
     }
 }
@@ -322,16 +332,20 @@ void SelectionPopup::insert(KPopupMenu *popup, Selection sel, bool showSubObject
         refVectorMenu->setCheckable(true);
         popup->insertItem(i18n("&Reference Vectors"), refVectorMenu);
         refVectorMenu->insertItem(i18n("Show Body Axes"), baseId + 25);
-        refVectorMenu->setItemChecked(baseId + 25, sel.body()->getVisibleReferenceMarks() & Body::BodyAxes);
+        refVectorMenu->setItemChecked(baseId + 25, sel.body()->findReferenceMark("body axes"));
         refVectorMenu->insertItem(i18n("Show Frame Axes"), baseId + 26);
-        refVectorMenu->setItemChecked(baseId + 26, sel.body()->getVisibleReferenceMarks() & Body::FrameAxes);
+        refVectorMenu->setItemChecked(baseId + 26, sel.body()->findReferenceMark("frame axes"));
         refVectorMenu->insertItem(i18n("Show Sun Direction"), baseId + 27);
-        refVectorMenu->setItemChecked(baseId + 27, sel.body()->getVisibleReferenceMarks() & Body::SunDirection);
+        refVectorMenu->setItemChecked(baseId + 27, sel.body()->findReferenceMark("sun direction"));
         refVectorMenu->insertItem(i18n("Show Velocity Vector"), baseId + 28);
-        refVectorMenu->setItemChecked(baseId + 28, sel.body()->getVisibleReferenceMarks() & Body::VelocityVector);
+        refVectorMenu->setItemChecked(baseId + 28, sel.body()->findReferenceMark("velocity vector"));
+        refVectorMenu->insertItem(i18n("Show Planetographic Grid"), baseId + 29);
+        refVectorMenu->setItemChecked(baseId + 29, sel.body()->findReferenceMark("planetographic grid"));
+        refVectorMenu->insertItem(i18n("Show Terminator"), baseId + 30);
+        refVectorMenu->setItemChecked(baseId + 30, sel.body()->findReferenceMark("terminator"));
     }
 
-    baseId += 30;
+    baseId += 31;
 
     if (showSubObjects && sel.body() != NULL)
     {
@@ -341,12 +355,12 @@ void SelectionPopup::insert(KPopupMenu *popup, Selection sel, bool showSubObject
             if (!altSurfaces->empty())
             {
                 KPopupMenu *surfaces = new KPopupMenu(this);
-                surfaces->insertItem(i18n("Normal"), locBaseId + 30);
+                surfaces->insertItem(i18n("Normal"), locBaseId + 31);
                 int j=0;
                 for (std::vector<std::string>::const_iterator i = altSurfaces->begin();
                      i < altSurfaces->end(); i++, j++)
                 {
-                    surfaces->insertItem(QString((*i).c_str()), locBaseId + 31 + j);
+                    surfaces->insertItem(QString((*i).c_str()), locBaseId + 32 + j);
                 }
                 baseId += 7 + j;
                 popup->insertItem(i18n("&Alternate Surfaces"), surfaces);
