@@ -2,7 +2,7 @@
 // Microsoft Visual C++ generated include file.
 // Used by celestia.rc
 //
-#define VERSION_STRING                  "1.5.1"
+#define VERSION_STRING                  "1.6.0"
 #define IDR_MENU1                       101
 #define IDD_ABOUT                       102
 #define IDI_CELESTIA_ICON               103
@@ -119,6 +119,7 @@
 #define IDC_LABELNEBULAE                1074
 #define IDC_LABELOPENCLUSTERS           1075
 #define IDC_LABELCONSTELLATIONSLATIN    1076
+#define IDC_SHOWOPENCLUSTERS                 1077
 #define IDCOMPUTE                       1078
 #define IDC_DATEFROM                    1079
 #define IDC_ECLIPSEFROM                 1079
@@ -134,6 +135,12 @@
 #define IDC_ECLIPSETYPE                 1089
 #define IDC_ECLIPSETARGET               1090
 #define IDC_SHOWCLOUDSHADOWS            1091
+#define IDC_LABELDWARFPLANETS                1092
+#define IDC_LABELMINORMOONS                  1093
+#define IDC_DWARFPLANETORBITS                1094
+#define IDC_MINORMOONORBITS                  1095
+#define IDC_LABELGLOBULARS           1096
+#define IDC_SHOWGLOBULARS                 1097
 #define IDC_SHOW_CITIES                 1100
 #define IDC_SHOW_OBSERVATORIES          1101
 #define IDC_SHOW_LANDING_SITES          1102
@@ -142,8 +149,12 @@
 #define IDC_SHOW_CRATERS                1105
 #define IDC_SHOW_VALLES                 1106
 #define IDC_SHOW_TERRAE                 1107
-#define IDC_SHOW_OTHERS                 1108
-#define IDC_LABELFEATURES               1109
+#define IDC_SHOW_VOLCANOES                 1108
+#define IDC_SHOW_OTHERS                 1109
+#define IDC_LABELFEATURES               1110
+#define IDC_SHOWHORIZONGRID           1111
+#define IDC_SHOWGALACTICGRID           1112
+#define IDC_SHOWECLIPTICGRID           1113
 #define IDC_SLIDER_FEATURE_SIZE         1150
 #define IDC_EDIT_FEATURE_SIZE           1151
 #define IDC_CELESTIALINK                1152
@@ -160,6 +171,9 @@
 #define ID_QUIT                         40008
 #define ID_FILE_EXIT                    40009
 #define ID_HELP_ABOUT                   40010
+#define ID_RENDER_TEXTURERES_LOW     40011
+#define ID_RENDER_TEXTURERES_MEDIUM      40012
+#define ID_RENDER_TEXTURERES_HIGH   40013
 #define ID_RENDER_AMBIENTLIGHT_NONE     40014
 #define ID_RENDER_AMBIENTLIGHT_LOW      40015
 #define ID_RENDER_AMBIENTLIGHT_MEDIUM   40016
@@ -219,6 +233,8 @@
 #define ID_RENDER_FRAME_AXES            40084
 #define ID_RENDER_SUN_DIRECTION         40085
 #define ID_RENDER_VELOCITY_VECTOR       40086
+#define ID_RENDER_PLANETOGRAPHIC_GRID   40087
+#define ID_RENDER_TERMINATOR            40088
 #define ID_BOOKMARKS_FIRSTBOOKMARK      41000
 #define ID_FIRST_SCRIPT                 42000
 
