@@ -65,6 +65,15 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_SHOWCELESTIALGRID:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowCelestialSphere);
             break;
+        case IDC_SHOWHORIZONGRID:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowHorizonGrid);
+            break;
+        case IDC_SHOWGALACTICGRID:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowGalacticGrid);
+            break;
+        case IDC_SHOWECLIPTICGRID:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowEclipticGrid);
+            break;
         case IDC_SHOWCLOUDS:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowCloudMaps);
             break;
@@ -80,9 +89,15 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_SHOWGALAXIES:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowGalaxies);
             break;
+        case IDC_SHOWGLOBULARS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowGlobulars);
+            break;
         case IDC_SHOWNEBULAE:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowNebulae);
             break;
+        case IDC_SHOWOPENCLUSTERS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowOpenClusters);
+            break;
         case IDC_SHOWNIGHTSIDELIGHTS:
             renderer->setRenderFlags(renderFlags ^ Renderer::ShowNightMaps);
             break;
@@ -92,12 +107,18 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_PLANETORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Planet);
             break;
+        case IDC_DWARFPLANETORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::DwarfPlanet);
+            break;
         case IDC_STARORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Stellar);
             break;
         case IDC_MOONORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Moon);
             break;
+        case IDC_MINORMOONORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::MinorMoon);
+            break;
         case IDC_ASTEROIDORBITS:
             renderer->setOrbitMask(orbitMask ^ Body::Asteroid);
             break;
@@ -134,18 +155,27 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
         case IDC_LABELGALAXIES:
             renderer->setLabelMode(labelMode ^ Renderer::GalaxyLabels);
             break;
-		case IDC_LABELNEBULAE:
-			renderer->setLabelMode(labelMode ^ Renderer::NebulaLabels);
-			break;
-		case IDC_LABELOPENCLUSTERS:
-			renderer->setLabelMode(labelMode ^ Renderer::OpenClusterLabels);
-			break;
+		    case IDC_LABELGLOBULARS:
+			      renderer->setLabelMode(labelMode ^ Renderer::GlobularLabels);
+			      break;
+		    case IDC_LABELNEBULAE:
+			      renderer->setLabelMode(labelMode ^ Renderer::NebulaLabels);
+			      break;
+		    case IDC_LABELOPENCLUSTERS:
+			      renderer->setLabelMode(labelMode ^ Renderer::OpenClusterLabels);
+			      break;
         case IDC_LABELPLANETS:
             renderer->setLabelMode(labelMode ^ Renderer::PlanetLabels);
             break;
+        case IDC_LABELDWARFPLANETS:
+            renderer->setLabelMode(labelMode ^ Renderer::DwarfPlanetLabels);
+            break;
         case IDC_LABELMOONS:
             renderer->setLabelMode(labelMode ^ Renderer::MoonLabels);
             break;
+        case IDC_LABELMINORMOONS:
+            renderer->setLabelMode(labelMode ^ Renderer::MinorMoonLabels);
+            break;
         case IDC_LABELSTARS:
             renderer->setLabelMode(labelMode ^ Renderer::StarLabels);
             break;
@@ -270,6 +300,12 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
         (renderFlags & Renderer::ShowAtmospheres)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCELESTIALGRID, BM_SETCHECK,
         (renderFlags & Renderer::ShowCelestialSphere)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWHORIZONGRID, BM_SETCHECK,
+        (renderFlags & Renderer::ShowHorizonGrid)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWGALACTICGRID, BM_SETCHECK,
+        (renderFlags & Renderer::ShowGalacticGrid)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWECLIPTICGRID, BM_SETCHECK,
+        (renderFlags & Renderer::ShowEclipticGrid)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCLOUDS, BM_SETCHECK,
         (renderFlags & Renderer::ShowCloudMaps)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCLOUDSHADOWS, BM_SETCHECK,
@@ -280,13 +316,19 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
         (renderFlags & Renderer::ShowEclipseShadows)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWGALAXIES, BM_SETCHECK,
         (renderFlags & Renderer::ShowGalaxies)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWGLOBULARS, BM_SETCHECK,
+        (renderFlags & Renderer::ShowGlobulars)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWNEBULAE, BM_SETCHECK,
         (renderFlags & Renderer::ShowNebulae)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWOPENCLUSTERS, BM_SETCHECK,
+        (renderFlags & Renderer::ShowOpenClusters)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWNIGHTSIDELIGHTS, BM_SETCHECK,
         (renderFlags & Renderer::ShowNightMaps)? BST_CHECKED:BST_UNCHECKED, 0);
     dlgCheck(hDlg, IDC_SHOWORBITS,       renderFlags, Renderer::ShowOrbits);
     dlgCheck(hDlg, IDC_PLANETORBITS,     orbitMask,   Body::Planet);
+    dlgCheck(hDlg, IDC_DWARFPLANETORBITS,     orbitMask,   Body::DwarfPlanet);
     dlgCheck(hDlg, IDC_MOONORBITS,       orbitMask,   Body::Moon);
+    dlgCheck(hDlg, IDC_MINORMOONORBITS,       orbitMask,   Body::MinorMoon);
     dlgCheck(hDlg, IDC_ASTEROIDORBITS,   orbitMask,   Body::Asteroid);
     dlgCheck(hDlg, IDC_COMETORBITS,      orbitMask,   Body::Comet);
     dlgCheck(hDlg, IDC_SPACECRAFTORBITS, orbitMask,   Body::Spacecraft);
@@ -308,11 +350,14 @@ void ViewOptionsDialog::SetControls(HWND hDlg)
     SendDlgItemMessage(hDlg, IDC_LABELCONSTELLATIONSLATIN, BM_SETCHECK,
                        ((labelMode & Renderer::I18nConstellationLabels) == 0) ? BST_CHECKED : BST_UNCHECKED, 0);
     dlgCheck(hDlg, IDC_LABELGALAXIES,   labelMode, Renderer::GalaxyLabels);
-	dlgCheck(hDlg, IDC_LABELNEBULAE,    labelMode, Renderer::NebulaLabels);
-	dlgCheck(hDlg, IDC_LABELOPENCLUSTERS, labelMode, Renderer::OpenClusterLabels);
+	  dlgCheck(hDlg, IDC_LABELGLOBULARS, labelMode, Renderer::GlobularLabels);
+	  dlgCheck(hDlg, IDC_LABELNEBULAE,    labelMode, Renderer::NebulaLabels);
+	  dlgCheck(hDlg, IDC_LABELOPENCLUSTERS, labelMode, Renderer::OpenClusterLabels);
     dlgCheck(hDlg, IDC_LABELSTARS,      labelMode, Renderer::StarLabels);
     dlgCheck(hDlg, IDC_LABELPLANETS,    labelMode, Renderer::PlanetLabels);
+    dlgCheck(hDlg, IDC_LABELDWARFPLANETS,    labelMode, Renderer::DwarfPlanetLabels);
     dlgCheck(hDlg, IDC_LABELMOONS,      labelMode, Renderer::MoonLabels);
+    dlgCheck(hDlg, IDC_LABELMINORMOONS,      labelMode, Renderer::MinorMoonLabels);
     dlgCheck(hDlg, IDC_LABELASTEROIDS,  labelMode, Renderer::AsteroidLabels);
     dlgCheck(hDlg, IDC_LABELCOMETS,     labelMode, Renderer::CometLabels);
     dlgCheck(hDlg, IDC_LABELSPACECRAFT, labelMode, Renderer::SpacecraftLabels);
